#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Copyright (C) 2012 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""

import unittest
import numpy as np
from LFPy import lfpcalc

class testLfpCalc(unittest.TestCase):
    """
    test class LFPy.lfpcalc
    """
    def test_return_dist_from_segment(self):
        cell = DummyCell()
        dist, clostest_point = lfpcalc.return_dist_from_segments(
                                           cell.xstart, cell.ystart,
                                           cell.zstart, cell.xend,
                                           cell.yend, cell.zend, [1, 10, 0])
        np.testing.assert_equal([10], dist)
        np.testing.assert_equal([1, 0, 0], clostest_point.T[0])

        dist, clostest_point = lfpcalc.return_dist_from_segments(
                                           cell.xstart, cell.ystart,
                                           cell.zstart, cell.xend,
                                           cell.yend, cell.zend, [-1, 10, 0])
        np.testing.assert_equal([np.sqrt(101)], dist)
        np.testing.assert_equal([0, 0, 0], clostest_point.T[0])


    def test_calc_lfp_pointsource(self):
        """Test that function calc_lfp_pointsource
        reproduces analytic formula"""
        sigma = 0.3
        cell = DummyCell()
        np.testing.assert_equal(1./(4*np.pi*sigma),
                                lfpcalc.calc_lfp_pointsource(cell,
                                x=0.5, y=0, z=1, sigma=sigma,
                                r_limit=cell.diam/2))

    def test_calc_lfp_pointsource_moi_homogeneous(self):
        """
        Test that slice where all layers have same conductivity reproduces
        in vivo results.
        """
        sigma_T = 0.3
        sigma_G = 0.3
        sigma_S = 0.3
        h = 300
        steps = 20
        cell = DummyCell()
        cell.zmid[0] = h/2
        cell.zstart[0] = h/2
        cell.zend[0] = h/2

        in_vivo = lfpcalc.calc_lfp_pointsource(cell,
                                x=0.5, y=0, z=1, sigma=sigma_T,
                                r_limit=cell.diam/2)
        in_vitro = lfpcalc.calc_lfp_pointsource_moi(cell,
                                x=0.5, y=0, z=1, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_equal(in_vivo, in_vitro)


    def test_calc_lfp_moi_ecog(self):
        """
        Test that LFPy ECoG scenario gives expected analytical result
        """

        sigma_T = 0.3
        sigma_G = 0.3
        sigma_S = 1.5
        h = 5000
        steps = 20
        cell = DummyCell()
        cell.zmid[0] = h - 50
        cell.zstart[0] = h - 50
        cell.zend[0] = h - 50

        cell.xmid[0] = 0
        cell.xstart[0] = 0
        cell.xend[0] = 0

        source_scaling = (sigma_T - sigma_S) / (sigma_S + sigma_T)

        z = h - 20  # Recording position z <= h, z != cell.zmid[0]

        analytic = cell.imem[0] / (4 * np.pi * sigma_T) * (
                   1 / np.abs(z - cell.zmid[0]) +  # real source
                   source_scaling / np.abs(z - (2 * h - cell.zmid[0]))  # image source
                   )

        moi_method_lfpy = lfpcalc.calc_lfp_pointsource_moi(cell,
                                x=0., y=0, z=z, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_equal(analytic, moi_method_lfpy)



    def test_calc_lfp_pointsource_moi_too_close(self):
        """
        Very close to point source, in vivo and in vitro have similar results,
        e.g., the positions should be adjusted similarly.
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 2000
        steps = 20
        cell = DummyCell()
        cell.zmid[0] = h/2
        cell.zstart[0] = h/2
        cell.zend[0] = h/2

        in_vivo = lfpcalc.calc_lfp_pointsource(cell,
                                x=0.5, y=0, z=h/2, sigma=sigma_T,
                                r_limit=cell.diam/2)
        in_vitro = lfpcalc.calc_lfp_pointsource_moi(cell,
                                x=0.5, y=0, z=h/2, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(in_vivo, in_vitro, 4)

    def test_calc_lfp_linesource_moi_too_close(self):
        """
        Very close to point source, in vivo and in vitro have similar results,
        e.g., the positions should be adjusted similarly.
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 0.3
        h = 200

        steps = 20
        cell = DummyCell()
        cell.zstart[0] = 0.0
        cell.zend[0] = 0.0

        in_vivo = lfpcalc.calc_lfp_linesource(cell,
                                x=0.5, y=0, z=0, sigma=sigma_T,
                                r_limit=cell.diam/2)
        in_vitro = lfpcalc.calc_lfp_linesource_moi(cell,
                                x=0.5, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(2*in_vivo, in_vitro, 4)

    def test_calc_lfp_soma_as_point_moi_too_close(self):
        """
        Very close to point source, in vivo and in vitro have similar results,
        e.g., the positions should be adjusted similarly.
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 2000
        steps = 20
        cell = DummyCell()
        cell.zmid[0] = 0
        cell.zstart[0] = 0
        cell.zend[0] = 0

        in_vivo = lfpcalc.calc_lfp_soma_as_point(cell,
                                x=0.0, y=0, z=0, sigma=sigma_T,
                                r_limit=cell.diam/2)
        in_vitro = lfpcalc.calc_lfp_soma_as_point_moi(cell,
                                x=0.0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(2*in_vivo, in_vitro, 4)


    def test_calc_lfp_linesource_too_close(self):
        """
        Test that calc_lfp_linesource method does not give infinite potential
        """
        sigma_T = 0.3
        cell = DummyCell()
        cell.zmid[0] = 0.0
        cell.zstart[0] = 0.0
        cell.zend[0] = 0.0

        in_vivo = lfpcalc.calc_lfp_linesource(cell,
                                x=0.5, y=0.0, z=0, sigma=sigma_T,
                                r_limit=cell.diam/2)[0]
        np.testing.assert_array_less(in_vivo, 1e12)

    def test_calc_lfp_pointsource_moi_doubling(self):
        """
        Test that slice with zero-conductivity MEA region (z<0) has twice
        the potential as in vivo case at MEA electrode plane
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 0.3
        h = 200
        steps = 3

        cell = DummyCell()
        cell.zstart[0] = 50
        cell.zmid[0] = 50
        cell.zend[0] = 50

        in_vivo = lfpcalc.calc_lfp_pointsource(cell,
                                x=50., y=0, z=0, sigma=sigma_T,
                                r_limit=cell.diam/2)
        in_vitro = lfpcalc.calc_lfp_pointsource_moi(cell,
                                x=50, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(2 * in_vivo, in_vitro, decimal=9)

    def test_calc_lfp_linesource_moi_doubling(self):
        """
        Test that slice with zero conductivity in MEA region (z<0) has twice
        the potential as in vivo case at MEA electrode plane
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 0.3
        h = 200
        steps = 3

        cell = DummyCell()
        cell.zstart[0] = 50
        cell.zmid[0] = 50
        cell.zend[0] = 50

        in_vivo = lfpcalc.calc_lfp_linesource(cell,
                                x=50., y=0, z=0, sigma=sigma_T,
                                r_limit=cell.diam/2)
        in_vitro = lfpcalc.calc_lfp_linesource_moi(cell,
                                x=50, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(2*in_vivo, in_vitro, decimal=9)

    def test_calc_lfp_soma_as_point_moi_doubling(self):
        """
        Test that slice with zero conductivity in MEA region (z<0) has twice
        the potential as in vivo case at MEA electrode plane
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 0.3
        h = 200
        steps = 3

        cell = DummyCell()
        cell.zstart[0] = 50
        cell.zmid[0] = 50
        cell.zend[0] = 50

        in_vivo = lfpcalc.calc_lfp_soma_as_point(cell,
                                x=50., y=0, z=0, sigma=sigma_T,
                                r_limit=cell.diam/2)
        in_vitro = lfpcalc.calc_lfp_soma_as_point_moi(cell,
                                x=50, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(2*in_vivo, in_vitro, decimal=9)


    def test_calc_lfp_pointsource_moi_saline_effect(self):
        """
        Test that the saline bath decreases signal as expected
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 200
        steps = 20

        cell = DummyCell()
        cell.zstart[0] = 100
        cell.zmid[0] = 100
        cell.zend[0] = 100

        with_saline = lfpcalc.calc_lfp_pointsource_moi(cell,
                                x=0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        without_saline = lfpcalc.calc_lfp_pointsource_moi(cell,
                                x=0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_T,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_array_less(with_saline, without_saline)

    def test_calc_lfp_linesource_moi_saline_effect(self):
        """
        Test that the saline bath decreases signal as expected
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 200
        steps = 20

        cell = DummyCell()
        cell.zstart[0] = 100
        cell.zmid[0] = 100
        cell.zend[0] = 100


        with_saline = lfpcalc.calc_lfp_linesource_moi(cell,
                                x=0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        without_saline = lfpcalc.calc_lfp_linesource_moi(cell,
                                x=0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_T,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_array_less(with_saline, without_saline)

    def test_calc_lfp_soma_as_point_moi_saline_effect(self):
        """
        Test that the saline bath decreases signal as expected
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 200
        steps = 20

        cell = DummyCell()
        cell.zstart[0] = 100
        cell.zmid[0] = 100
        cell.zend[0] = 100

        with_saline = lfpcalc.calc_lfp_soma_as_point_moi(cell,
                                x=0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        without_saline = lfpcalc.calc_lfp_soma_as_point_moi(cell,
                                x=0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_T,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_array_less(with_saline, without_saline)



    def test_calc_lfp_pointsource_moi_20steps(self):
        """
        Test that the calc_lfp_pointsource_moi reproduces previously known
        nummerical value
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 200
        steps = 20

        correct = 0.00108189

        cell = DummyCell()
        cell.zmid[0] = 110
        cell.xmid[0] = -100

        calculated = lfpcalc.calc_lfp_pointsource_moi(cell,
                                x=100, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(correct, calculated, 5)


    def test_calc_lfp_linesource_moi_20steps(self):
        """
        Test that the calc_lfp_linesource_moi reproduces previously known
        nummerical value
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 200
        steps = 20

        correct = 0.00246539

        cell = DummyCell()
        cell.zstart[0] = 0
        cell.zend[0] = 110
        cell.xstart[0] = -100
        cell.xend[0] = 50

        calculated = lfpcalc.calc_lfp_linesource_moi(cell,
                                x=100, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(correct, calculated, 5)

    def test_calc_lfp_soma_as_point_moi_20steps(self):
        """
        Test that the calc_lfp_soma_as_point_moi reproduces previously known
        nummerical value
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 200
        steps = 20

        correct = 0.00108189

        cell = DummyCell()
        cell.zmid[0] = 110
        cell.xmid[0] = -100

        calculated = lfpcalc.calc_lfp_soma_as_point_moi(cell,
                                x=100, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(correct, calculated, 5)



    def test_calc_lfp_pointsource_moi_infinite_slice(self):
        """
        Test that infinitely thick slice does not affect potential.
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 1e10
        steps = 20

        cell = DummyCell()
        cell.zstart[0] = 100
        cell.zmid[0] = 100
        cell.zend[0] = 100

        with_saline = lfpcalc.calc_lfp_pointsource_moi(cell,
                                x=0, y=0, z=50, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        without_saline = lfpcalc.calc_lfp_pointsource_moi(cell,
                                x=0, y=0, z=50, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_T,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(with_saline, without_saline)


    def test_calc_lfp_linesource_moi_infinite_slice(self):
        """
        Test that infinitely thick slice does not affect potential.
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 1e10
        steps = 20

        cell = DummyCell()
        cell.zstart[0] = 100
        cell.zmid[0] = 100
        cell.zend[0] = 100

        with_saline = lfpcalc.calc_lfp_linesource_moi(cell,
                                x=0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        without_saline = lfpcalc.calc_lfp_linesource_moi(cell,
                                x=0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_T,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(with_saline, without_saline)

    def test_calc_lfp_soma_as_point_moi_infinite_slice(self):
        """
        Test that infinitely thick slice does not affect potential.
        """
        sigma_T = 0.3
        sigma_G = 0.0
        sigma_S = 1.5
        h = 1e10
        steps = 20

        cell = DummyCell()
        cell.zstart[0] = 100
        cell.zmid[0] = 100
        cell.zend[0] = 100

        with_saline = lfpcalc.calc_lfp_soma_as_point_moi(cell,
                                x=0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_S,
                                r_limit=cell.diam/2, h=h, steps=steps)

        without_saline = lfpcalc.calc_lfp_soma_as_point_moi(cell,
                                x=0, y=0, z=0, sigma_T=sigma_T,
                                sigma_G=sigma_G, sigma_S=sigma_T,
                                r_limit=cell.diam/2, h=h, steps=steps)

        np.testing.assert_almost_equal(with_saline, without_saline)



    def test_calc_lfp_pointsource_anisotropic(self):

        sigma = [0.6, 0.3, 0.45]
        cell = DummyCell()
        cell.xmid = cell.ymid = cell.zmid = np.array([1.2])
        sigma_r = np.sqrt(sigma[1] * sigma[2] * 1.2**2
                        + sigma[0] * sigma[2] * 1.2**2
                        + sigma[0] * sigma[1] * 1.2**2)

        phi_analytic = 1./(4*np.pi * sigma_r)
        np.testing.assert_equal(phi_analytic,
                                lfpcalc.calc_lfp_pointsource_anisotropic(cell,
                                x=0, y=0, z=0, sigma=sigma,
                                r_limit=cell.diam/2))


    def test_deltaS_calc(self):
        cell = DummyCell()
        cell.yend[0] = 5
        ds = lfpcalc._deltaS_calc(cell.xstart, cell.xend,
                                  cell.ystart, cell.yend,
                                  cell.zstart, cell.zend)
        np.testing.assert_equal(ds, np.sqrt(26))



class DummyCell(object):
    """Cell like object with attributes for predicting extracellular potentials,
    but with:
        - 1 compartment
        - position in (0.5,0,0)
        - length 1
        - diam 1
        - current amplitude 1
        - 1 timestep
    """
    def __init__(self):
        self.imem = np.array([[1.]])
        self.xstart = np.array([0.])
        self.ystart = np.array([0.])
        self.zstart = np.array([0.])
        self.xmid = np.array([0.5])
        self.ymid = np.array([0.])
        self.zmid = np.array([0.])
        self.xend = np.array([1.])
        self.yend = np.array([0.])
        self.zend = np.array([0.])
        self.diam = np.array([1.])
        self.totnsegs = len(self.xmid)
    
    def get_idx(self, section="soma"):
        if section == "soma":
            return np.array([0])
        else:
            return np.array([])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Copyright (C) 2012 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""


import unittest
import numpy as np
import scipy.stats as st
import LFPy


class testInputGenerators(unittest.TestCase):
    """
    test LFPy.inputgenerators module
    """
    def test_get_activation_times_from_distribution(self):
        """test LFPy.inputgenerators.get_activation_times_from_distribution
        """
        n = 10
        tstart = 0.
        tstop = 1E4
        distribution = st.expon
        rvs_args = dict(loc=0, scale=100.)
        times = LFPy.inputgenerators.get_activation_times_from_distribution(
            n=n, tstart=tstart, tstop=tstop, rvs_args=rvs_args, maxiter=1E6
        )
        self.assertTrue(len(times) == n)
        for t in times:
            self.assertTrue((t.min() >= tstart) & (t.max() <= tstop))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Example plot for LFPy: Single-synapse contribution to the EEG

Execution:

    python example_EEG.py
    
Copyright (C) 2017 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
"""
import LFPy
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import colorbar

def plot_cell_to_ax(cell, ax, synidxs):
    for idx in range(cell.totnsegs):
        if idx == 0:
            ax.plot(cell.xmid[idx], cell.zmid[idx], 'ko')
        else:
            ax.plot([cell.xstart[idx], cell.xend[idx]],
                    [cell.zstart[idx], cell.zend[idx]], c='k')

    for synidx in synidxs:
        l, = ax.plot(cell.xmid[synidx], cell.zmid[synidx], '*',
                c="r", ms=10)
    ax.legend([l], ["Synapse"], frameon=False, bbox_to_anchor=[1, -0.1])

def plot_EEG_sphere(fig, eeg, x_eeg, y_eeg, z_eeg):
    from mpl_toolkits.mplot3d import Axes3D
    ax = fig.add_subplot(322, projection='3d',
                         title="Max EEG potential\nat 4-sphere surface")
    vmax = 6
    vmin = -vmax
    clr = lambda phi: plt.cm.PRGn((phi - vmin) / (vmax - vmin))
    clrs = clr(eeg)
    surf = ax.plot_surface(x_eeg.reshape(num_theta, num_phi),
                           y_eeg.reshape(num_theta, num_phi),
                           z_eeg.reshape(num_theta, num_phi),
                           rstride=1, cstride=1, facecolors=clrs,
                           linewidth=0, antialiased=False)

    #ax.set_aspect('equal') # Raises NotImplementedError in MPL currently
    ax.axis('off')
    ax.set_xlim3d(-65000, 65000)
    ax.set_ylim3d(-65000, 65000)
    ax.set_zlim3d(-65000, 65000)
    ax.view_init(10, 0)

    # colorbar
    cax = fig.add_axes([0.65, 0.75, 0.25, 0.01])
    m = plt.cm.ScalarMappable(cmap=plt.cm.PRGn)
    ticks = np.linspace(vmin, vmax, 5) # global normalization
    m.set_array(ticks)
    cbar = fig.colorbar(m, cax=cax,
                        extend='both', orientation='horizontal')
    cbar.outline.set_visible(False)
    cbar.set_ticks(ticks)
    cbar.set_label(r'$\phi$ (pV)', labelpad=1.)

if __name__ == '__main__':

    # four_sphere properties
    radii = [79000., 80000., 85000., 90000.]
    sigmas = [0.3, 1.5, 0.015, 0.3]
    rad_tol = 1e-2

    # simulate cell
    syn_loc = (0, 0, 1000)

    cell_params = {'morphology': 'morphologies/L5_Mainen96_LFPy.hoc',
                   'cm' : 1.0,                 # membrane capacitance
                   'Ra' : 150,                 # axial resistance
                   'tstart': 0.,
                   'passive' : True,           # switch on passive mechs
                   'nsegs_method' : 'lambda_f',# method for setting number of segments,
                   'lambda_f' : 100,           # segments are isopotential at this frequency
                   'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -70}, # passive params
                   'tstop': 40
                   }

    synapse_params = {'e': 0.,  # reversal potential
                      'syntype': 'ExpSyn',  # exponential synapse
                      'tau': 5.,  # synapse time constant
                      'weight': 0.001,  # 0.001, # synapse weight
                      'record_current': True  # record synapse current
                     }
    # create cell with parameters in dictionary
    cell = LFPy.Cell(**cell_params)
    pos = syn_loc
    synapse_params['idx'] = cell.get_closest_idx(x=pos[0], y=pos[1], z=pos[2])
    synapse = LFPy.Synapse(cell, **synapse_params)
    synapse.set_spike_times(np.array([5.]))

    cell.simulate(rec_imem=True,
                  rec_current_dipole_moment=True)

    # compute dipole
    P = cell.current_dipole_moment

    somapos = np.array([0., 0., 77500])
    r_soma_syns = [cell.get_intersegment_vector(idx0=0, idx1=i) for i in cell.synidx]
    r_mid = np.average(r_soma_syns, axis=0)
    r_mid = somapos + r_mid/2.

    eeg_coords_top = np.array([[0., 0., radii[3] - rad_tol]])
    four_sphere_top = LFPy.FourSphereVolumeConductor(radii, sigmas, eeg_coords_top)
    pot_db_4s_top = four_sphere_top.calc_potential(P, r_mid)
    eeg_top = np.array(pot_db_4s_top) * 1e9

    #measurement points
    # for nice plot use theta_step = 1 and phi_step = 1. NB: Long computation time.
    theta_step = 5
    phi_step = 5
    theta, phi_angle = np.mgrid[0.:180.:theta_step, 0.:360.+phi_step:phi_step]

    num_theta = theta.shape[0]
    num_phi = theta.shape[1]
    theta = theta.flatten()
    phi_angle = phi_angle.flatten()

    theta_r = np.deg2rad(theta)
    phi_angle_r = np.deg2rad(phi_angle)

    x_eeg = (radii[3] - rad_tol) * np.sin(theta_r) * np.cos(phi_angle_r)
    y_eeg = (radii[3] - rad_tol) * np.sin(theta_r) * np.sin(phi_angle_r)
    z_eeg = (radii[3] - rad_tol) * np.cos(theta_r)
    eeg_coords = np.vstack((x_eeg, y_eeg, z_eeg)).T

    # potential in 4S with db
    time_max = np.argmax(np.linalg.norm(P, axis=1))
    p = P[time_max, None]

    four_sphere = LFPy.FourSphereVolumeConductor(radii, sigmas, eeg_coords)

    pot_db_4s = four_sphere.calc_potential(p, r_mid)
    eeg = pot_db_4s.reshape(num_theta, num_phi)*1e9# from mV to pV

    P_mag = np.sqrt(P[:, 0]**2 + P[:, 1]**2 + P[:, 2]**2)

    plt.close('all')
    fig = plt.figure(figsize=[6, 8])
    fig.subplots_adjust(left=0.15, hspace=0.5, right=0.98, top=0.95)

    morph_ax = fig.add_subplot(321, aspect=1, frameon=False,
                               title="Cell and synapse",
                               xticks=[], yticks=[])
    plot_cell_to_ax(cell, morph_ax, cell.synidx)

    plot_EEG_sphere(fig, eeg, x_eeg, y_eeg, z_eeg)

    ax_p = fig.add_subplot(312, title="Current dipole moment", ylabel="nA$\cdot\mu$m")
    ax_p.plot(cell.tvec, P[:, 0], label="P$_x$")
    ax_p.plot(cell.tvec, P[:, 1], label="P$_y$")
    ax_p.plot(cell.tvec, P[:, 2], label="P$_z$")
    ax_p.axvline(cell.tvec[time_max], c='gray', ls='--')
    ax_p.legend(frameon=False, ncol=4, bbox_to_anchor=[1, -0.1])

    ax_eeg = fig.add_subplot(313, title="EEG at top", xlabel="Time (ms)",
                             ylabel='pV')
    ax_eeg.plot(cell.tvec, eeg_top[0, :], 'k')

    ax_eeg.axvline(cell.tvec[time_max], c='gray', ls='--')

    # plt.savefig('example_EEG.pdf')
    plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Simulation and plotting script reproducing figure 2 of:

Multimodal modeling of neural network activity: computing LFP, ECoG, EEG and MEG signals with LFPy2.0
Espen Hagen, Solveig Næss, Torbjørn V Ness, Gaute T Einevoll
bioRxiv 281717; doi: https://doi.org/10.1101/281717
'''
# import of modules
import LFPy
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
from mpl_toolkits.axisartist.axislines import SubplotZero
from matplotlib.collections import PolyCollection
import neuron

# set some plotting parameters
fontsize=14
titlesize=16
legendsize=12
plt.rcParams.update({
    'axes.xmargin': 0.0,
    'axes.ymargin': 0.0,
    'axes.labelsize': fontsize,
    'axes.titlesize': titlesize,
    'figure.titlesize': fontsize,
    'font.size': fontsize,
    'legend.fontsize': legendsize,
})

################################################################################
# Main script, set parameters and create cell, synapse and electrode objects
################################################################################

# clear out old section references in NEURON
neuron.h('forall delete_section()')

# parameters for Cell instance with simplified morphology
cellParameters = {
    'morphology' : 'simplemorpho_modded.hoc',
    'tstop' : 10.,      # sim duration
    'dt' : 2**-4,       # sim time step size
    'v_init' : -65,     # intitial voltage
    'cm' : 1.,          # membrane capacitance
    'Ra' : 150.,        # axial resistivity
    'passive' : True,   # turn on passive mechanism
    'passive_parameters' : {'g_pas' : 1./3E4,   # passive leak conductance
                            'e_pas' : -65.},    # leak reversal potential
    'pt3d' : True,      # keep morphological detail
}
# create cell, set alignment
cell = LFPy.Cell(**cellParameters)
cell.set_rotation(x=np.pi/2)

# synapse location index
synidx = cell.get_idx(['apic[3]'])[1]

# parameters for synapse instance
synapseParameters = {
    'idx' : synidx,
    'e' : 0,                                # reversal potential
    'syntype' : 'Exp2Syn',                   # synapse type
    'tau1' : 0.1,                              # syn. rise time constant
    'tau2' : 1.,                              # syn. decay time constant
    'weight' : 0.002,                        # syn. weight
    'record_current' : True                 # syn. current record
}
# create synapse and set activation time
synapse = LFPy.Synapse(cell, **synapseParameters)
synapse.set_spike_times(np.array([2.]))

# extracellular electrode parameters
dx = 1. # spatial resolution
x = np.arange(-100., 140.+dx, dx)
z = np.arange(-85., 255.+dx, dx)
X, Z = np.meshgrid(x, z)
electrodeParameters = {
    'x' : X.flatten(),
    'y' : np.zeros(X.size),
    'z' : Z.flatten(),
    'sigma' : 0.3,
    'method' : 'soma_as_point',
}
# instantiate electrode
electrode = LFPy.RecExtElectrode(**electrodeParameters)

# compute cell response, current dipole moment and extracellular potential
cell.simulate(electrode=electrode,
              rec_imem=True,
              rec_vmem=True,
              rec_current_dipole_moment=True)


# compute effective dipole location as the cell's 'center of gravity of areas'
R_cell = (cell.area * np.c_[cell.xmid, cell.ymid, cell.zmid].T / cell.area.sum()).sum(axis=1)

# compute the electric potential of the dipole using
# \phi = \frac{1}{4 pi \sigma} \frac{\mathbf{p} \cdot \hat\mathbf{R}}{R^2}
# where
# \hat\mathbf{R} = \frac{\mathbf{R}}{R}
dx_p = 10.
x_p = np.arange(-1200., 1200.+dx_p, dx_p)
z_p = np.arange(-1700., 1700.+dx_p, dx_p)
X_p, Z_p = np.meshgrid(x_p, z_p)
Y_p = np.zeros(X_p.shape)
R = np.c_[X_p.flatten(), Y_p.flatten(), Z_p.flatten()]
R_rel = R - R_cell
R_scalar = np.sqrt((R_rel**2).sum(axis=1))
phi_p = 1. / (4*np.pi*electrode.sigma) * np.dot(cell.current_dipole_moment,
                                                R_rel.T) / R_scalar**3 # (omega*m*nA*µm/µm^3=mV)
# mask out values in spatial locations in vicinity of the cell:
mask = np.zeros(phi_p.shape).astype(bool)
mask[:, R_scalar < 500.] = True
phi_p = np.ma.masked_array(phi_p, mask=mask)


# compute potential on dipole grid
electrodeParameters_p = {
    'x' : X_p.flatten(),
    'y' : np.zeros(X_p.size),
    'z' : Z_p.flatten(),
    'sigma' : 0.3,
    'method' : 'soma_as_point',
}
electrode_p = LFPy.RecExtElectrode(cell=cell, **electrodeParameters_p)
electrode_p.calc_lfp()
LFP_p = np.ma.masked_array(electrode_p.LFP, mask=mask.T==False)


# Compute the magnetic field strengt |\mathbf{H}| at locations corresponding
# the electrode grid from the dipole moment using the Biot-Savart law
# H = (p x R) / (4 pi u_0 |R|**3)
H = np.zeros((cell.current_dipole_moment.shape[0], R_rel.shape[0], 3))
for i, r in enumerate(R_rel):
    if R_scalar[i] > 500:
        H[:, i, :] = np.cross(cell.current_dipole_moment, r) / (4 * np.pi * np.sqrt((r**2).sum())**3)


# for locations within 500 um, compute the magnetic field from axial currents
i_axial, d_vectors, pos_vectors = cell.get_axial_currents_from_vmem()
inds = np.where(R_scalar < 500.)[0]
for i in inds:
    R_ = R[i, ]
    for i_, d_, r_ in zip(i_axial, d_vectors, pos_vectors):
        r_rel = R_ - r_
        H[:, i, :] += np.dot(i_.reshape((-1, 1)),
                             np.cross(d_, r_rel).reshape((1, -1))
                             ) / (4*np.pi*np.sqrt((r_rel**2).sum())**3) 

# set up four-sphere head model params
_theta = np.linspace(-np.pi/4, np.pi / 4, 9)
_x = 90000.*np.sin(_theta)
_y = np.zeros(_theta.size)
_z = 90000.*np.cos(_theta)
foursphereParams = {
    'radii' : [79000., 80000., 85000., 90000.], # shell radii
    'sigmas' : [0.3, 1.5, 0.015, 0.3],          # shell conductivity
    'r' : np.c_[_x, _y, _z],                    # contact coordinates
}
dipole_position = np.array([0, 0, 78000.])      # dipole location


#######################################
# Define some plotting helper functions
#######################################
def draw_lineplot(
        ax, data, dt=0.1,
        T=(0, 200),
        scaling_factor=1.,
        vlimround=None,
        label='local',
        scalebar=True,
        scalebarpos=0,
        scalebarbasis='log2',
        unit='mV',
        ylabels=True,
        color='r',
        ztransform=True,
        filter=False,
        filterargs=dict(N=2, Wn=0.02, btype='lowpass')):
    ''' draw some nice lines'''
    
    tvec = np.arange(data.shape[1])*dt
    try:
        tinds = (tvec >= T[0]) & (tvec <= T[1])
    except TypeError:
        print(data.shape, T)
        raise Exception
    
    # apply temporal filter
    if filter:
        b, a = ss.butter(**filterargs)
        data = ss.filtfilt(b, a, data, axis=-1)
    
    #subtract mean in each channel
    if ztransform:
        dataT = data.T - data.mean(axis=1)
        data = dataT.T

    zvec = -np.arange(data.shape[0])
    vlim = abs(data[:, tinds]).max()
    if vlimround is None:
        vlimround = 2.**np.round(np.log2(vlim)) / scaling_factor
    else:
        pass

    yticklabels=[]
    yticks = []
    
    for i, z in enumerate(zvec):
        if i == 0:
            ax.plot(tvec[tinds], data[i][tinds] / vlimround + z, lw=1,
                    rasterized=False, label=label, clip_on=False,
                    color=color)
        else: 
            ax.plot(tvec[tinds], data[i][tinds] / vlimround + z, lw=1,
                    rasterized=False, clip_on=False,
                    color=color)
        yticklabels.append('ch. %i' % (i+1))
        yticks.append(z)
     
    if scalebar:
        if scalebarbasis == 'log2':
            ax.plot([tvec[tinds][-1], tvec[tinds][-1]],
                    [-1-scalebarpos, -2-scalebarpos], lw=2, color=color, clip_on=False)
            ax.text(tvec[tinds][-1]+np.diff(T)*0.03, -1.5-scalebarpos,
                    '$2^{' + '{}'.format(int(np.log2(vlimround))) + '}$ ' + '{0}'.format(unit),
                    color=color, rotation='vertical',
                    va='center')
        elif scalebarbasis == 'log10':
            # recompute scale bar size to show it on scientific format
            vlimround10 = 10**np.round(np.log10(vlimround))
            if vlimround10 >= 1:
                vlimround10 = int(np.round(vlimround10))
            rescale = vlimround10 / vlimround
            ax.plot([tvec[tinds][-1], tvec[tinds][-1]],
                    np.array([0.5, -0.5])*rescale - 1.5 - scalebarpos,
                    lw=2, color=color, clip_on=False)
            ax.text(tvec[tinds][-1]+np.diff(T)*0.03, -1.5-scalebarpos,
                    '{0} '.format(vlimround10) + '{0}'.format(unit),
                    color=color, rotation='vertical',
                    va='center')

    ax.axis(ax.axis('tight'))
    ax.yaxis.set_ticks(yticks)
    if ylabels:
        ax.yaxis.set_ticklabels(yticklabels)
        ax.set_ylabel('channel', labelpad=0.1)
    else:
        ax.yaxis.set_ticklabels([])
    remove_axis_junk(ax, lines=['right', 'top'])
    ax.set_xlabel(r'time (ms)', labelpad=0.1)
    
    return vlimround


def remove_axis_junk(ax, lines=['right', 'top']):
    for loc, spine in ax.spines.items():
        if loc in lines:
            spine.set_color('none')            
    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.set_ticks_position('left')


#######################################
# Figure
#######################################

# plot and annotate
plt.close('all')
fig = plt.figure(figsize=(16, 12))
fig.subplots_adjust()
gs = GridSpec(9, 3, left=0.05, right=0.98, wspace=0.15, hspace=-0.2, bottom=0.05, top=0.95)
alphabet = 'ABCDEF'

# set up subplot panels
ax0 = fig.add_subplot(gs[:6, 0], aspect='equal') # LFP forward model ill.
ax0.axis('off')
ax0.set_title('extracellular potential')
ax1 = fig.add_subplot(gs[:6, 1], aspect='equal') # dipole moment ill.
ax1.axis('off')
ax1.set_title('extracellular potential')
ax2 = fig.add_subplot(gs[:6, 2], aspect='equal') # dipole moment ill.
ax2.axis('off')
ax2.set_title('magnetic field')
# ax3 = fig.add_subplot(gs[0, 3], aspect='equal')             # spherical shell model ill.
# ax3.set_title('4-sphere volume conductor')
# ax4 = fig.add_subplot(gs[1, 3],
                      # aspect='equal'
                      # )                 # MEG/EEG forward model ill.
# ax4.set_title('EEG and MEG signal detection')

ax3 = SubplotZero(fig, gs[7:, 0])
fig.add_subplot(ax3)
ax3.set_title('4-sphere volume conductor', verticalalignment='bottom')
ax4 = fig.add_subplot(gs[7:, 1]) # EEG
ax4.set_title('scalp electric potential $\phi_\mathbf{p}(\mathbf{r})$')
ax5 = fig.add_subplot(gs[7:, 2], sharey=ax4) # MEG
# ax5.set_title('scalp magnetic field')

#morphology - line sources for panels A and B
zips = []
xz = cell.get_idx_polygons()
for x, z in xz:
    zips.append(list(zip(x, z)))
for ax in [ax0]:
    polycol = PolyCollection(zips,
                             linewidths=(0.5),
                             edgecolors='k',
                             facecolors='none',
                             zorder=-5)
    ax.add_collection(polycol)

    #morphology mid points
    ax.plot(cell.xmid, cell.zmid, 'o', mec='none', mfc='k',
             markersize=3, zorder=0)

    # mark synapse location
    ax.plot(synapse.x+5, synapse.z, '> ', mec='r', mfc='r', markersize=10, zorder=1)

for ax in [ax1, ax2]:
    polycol = PolyCollection(zips,
                             linewidths=(0.5),
                             edgecolors='gray',
                             facecolors='gray',
                             zorder=-5)
    ax.add_collection(polycol)

    # mark synapse location
    ax.plot(synapse.x+5, synapse.z, '> ', mec='r', mfc='r', markersize=5, zorder=1)

#morphology - offset with pt3d info for panel A
zips = []
offset = 0.
for x, z in cell.get_pt3d_polygons():
    zips.append(list(zip(x-offset, z+offset)))
polycol = PolyCollection(zips,
                         edgecolors='none',
                         facecolors='gray',
                         alpha=0.5,
                         zorder=-6)
ax0.add_collection(polycol)

#some extracellular position and annotation
ax0.plot(-80, 30, 'o',
        markeredgecolor='none', markerfacecolor='b', markersize=5, zorder=0, clip_on=False)
ax0.text(-80, 40, r'$\phi({\bf r}, t)$', horizontalalignment='center')
idx = cell.get_idx('apic')[2]
ax0.plot([cell.xmid[idx], -80], [cell.zmid[idx], 30], '-.k', linewidth=1.0, clip_on=False)
ax0.fill(xz[idx][0], xz[idx][1], edgecolor='none', facecolor='k', zorder=-7)
ax0.text(cell.xmid[idx]+5, cell.zmid[idx], r'$I_n^\mathrm{m}({\bf r}_n, t)$', verticalalignment='center')
ax0.text(cell.xmid[idx]-40, cell.zmid[idx]-10, r'$|{\bf r}-{\bf r}_n|$', horizontalalignment='center')



# draw LFPs at t=|i_syn|_max
t_max = abs(synapse.i) == abs(synapse.i).max()
vmax = 2
im = ax0.pcolormesh(X, Z, electrode.LFP[:, t_max].reshape(X.shape)*1E3,
                  cmap=plt.get_cmap('PRGn', 51), zorder=-10,
                  vmin=-vmax,
                  vmax=vmax,
                  rasterized=True)
bbox = np.array(ax0.get_position())
cax = fig.add_axes([bbox[0][0]+(bbox[1][0]-bbox[0][0])/4, bbox[0][1], (bbox[1][0]-bbox[0][0])/2, 0.015])
axcb = fig.colorbar(im, cax=cax, orientation='horizontal')
axcb.outline.set_visible(False)
axcb.set_label(r'$\phi$ ($\mu$V)', labelpad=0)
axcb.set_ticks([-vmax, 0, vmax])

# plot arrow representing dipole moment magnitude and direction
ax0.annotate("", #r"$\mathbf{p}(I_n^{(\mathrm{m})}(t), \mathbf{r}_n)$",
            xy=(R_cell[0], R_cell[2]),
            xytext=(R_cell[0] + cell.current_dipole_moment[t_max][0, 0]*5,
                    R_cell[2] + cell.current_dipole_moment[t_max][0, 2]*5),
            arrowprops=dict(arrowstyle="<-", lw=3, color='k', shrinkA=0, shrinkB=0
                            ),
            zorder=100)
ax0.text(R_cell[0] + cell.current_dipole_moment[t_max][0, 0] * 5 / 2 + 1,
         R_cell[2] + cell.current_dipole_moment[t_max][0, 2] * 5/ 2,
         r"$\mathbf{p}(I_n^\mathrm{m}(t), \mathbf{r}_n)$")


# create axes for synapse input current
synax = fig.add_axes([0.29, 0.82, bbox[1][0]-0.27, 0.08])
synax.plot(cell.tvec, synapse.i, 'r', lw=1)
synax.set_xticks([0, 5, 10])
synax.set_xticklabels([])
synax.set_title('I')
# synax.set_xlabel('time (ms)')
synax.set_ylabel(r'$i_\mathrm{syn}(t)$ (nA)')

# axes for somatic voltage
vax = fig.add_axes([0.29, 0.70, bbox[1][0]-0.27, 0.08])
vax.plot(cell.tvec, cell.somav, 'r', lw=1)
vax.set_xticks([0, 5, 10])
vax.set_xticklabels([])
vax.set_title('II')
# synax.set_xlabel('time (ms)')
vax.set_ylabel(r'$V_\mathrm{soma}(t)$ (mV)')


# create axes for extracellular potential
lfpax = fig.add_axes([0.29, 0.58, bbox[1][0]-0.27, 0.08])
lfpax.plot(cell.tvec, electrode.LFP[(electrode.x == -80) & (electrode.z == 30), ].ravel()*1E3, 'b', lw=1)
lfpax.set_xticks([0, 5, 10])
lfpax.set_xticklabels([])
lfpax.set_title('III')
# lfpax.set_xlabel('time (ms)')
lfpax.set_ylabel(r'$\phi(\mathbf{r}, t)$ ($\mu$V)')

# create axes for current dipole moment
pax = fig.add_axes([0.29, 0.46, bbox[1][0]-0.27, 0.08])
for i, x in enumerate(cell.current_dipole_moment[:, ::2].T):
    pax.plot(cell.tvec, x*1E-3, label=r'$\mathbf{\hat{u}=\hat{%s}}$' % ('xz'[i]), lw=1)  # nA um -> 1E-3 nA m unit conversion
pax.set_xticks([0, 5, 10])
pax.set_xlabel('time (ms)')
pax.set_ylabel(r'$\mathbf{p} \cdot \hat{\mathbf{u}}}$ ($10^{-3}$ nA m)')
pax.set_title('IV')
pax.legend(loc=8, bbox_to_anchor=(0.5, -1.25))


# scale bars
ax0.plot([60, 60], [-80, -70], 'k', lw=1, clip_on=False)
ax0.text(62, -80, r'$10 \mu$m', fontsize=12)

# axis cross
ax0.annotate("", xy=(-90, -80), xycoords='data', xytext=(-70, -80), textcoords='data',
             arrowprops=dict(arrowstyle="<|-", connectionstyle="arc3,rad=0", facecolor='black'))
ax0.annotate("", xy=(-90, -80), xycoords='data', xytext=(-90, -60), textcoords='data',
             arrowprops=dict(arrowstyle="<|-", connectionstyle="arc3,rad=0", facecolor='black'))
ax0.text(-70, -90, 'x', ha='right')
ax0.text(-100, -70, 'z')


# Plot dipole moment potential at t=|i_syn|_max
# t_max = abs(synapse.i) == abs(synapse.i).max()
vmax = 5.
im = ax1.pcolormesh(X_p, Z_p, phi_p[t_max, ].reshape(X_p.shape)*1E6,
                  cmap=plt.get_cmap('PRGn', 51), zorder=-10,
                  vmin=-vmax,
                  vmax=vmax,
                  rasterized=True)
_ = ax1.pcolormesh(X_p, Z_p, LFP_p.T[t_max, ].reshape(X_p.shape)*1E6,
                  cmap=plt.get_cmap('PRGn', 51), zorder=-10,
                  vmin=-vmax,
                  vmax=vmax,
                  rasterized=True)
# draw circle between "close field" and "far field"
phi = np.linspace(0, 2*np.pi, 37)
r = 500
x = r*np.cos(phi) + R_cell[0]
y = r*np.sin(phi) + R_cell[2]
ax1.plot(x, y, ':w', lw=2)
ax1.text(R_cell[0], r+R_cell[2]+10, r'$\phi_\mathbf{p}(\mathbf{r}, t)$', ha='center', va='bottom', color='w')
ax1.text(R_cell[0], r+R_cell[2]-10, r'$\phi(\mathbf{r}, t)$', ha='center', va='top', color='w')

bbox = np.array(ax1.get_position())
cax = fig.add_axes([bbox[0][0]+(bbox[1][0]-bbox[0][0])/4, bbox[0][1], (bbox[1][0]-bbox[0][0])/2, 0.015])
axcb = fig.colorbar(im, cax=cax, orientation='horizontal')
axcb.outline.set_visible(False)
axcb.set_label(r'$\phi$ (nV)', labelpad=0)
axcb.set_ticks([-vmax, 0, vmax])

# scale bars
ax1.plot([800, 800], [-1650, -1550], 'k', lw=1, clip_on=False)
ax1.text(820, -1650, r'$100 \mu$m', fontsize=12)

# axis cross
ax1.annotate("", xy=(-1100, -1600), xycoords='data', xytext=(-900, -1600), textcoords='data',
             arrowprops=dict(arrowstyle="<|-", connectionstyle="arc3,rad=0", facecolor='black'))
ax1.annotate("", xy=(-1100, -1600), xycoords='data', xytext=(-1100, -1400), textcoords='data',
             arrowprops=dict(arrowstyle="<|-", connectionstyle="arc3,rad=0", facecolor='black'))
ax1.text(-900, -1700, 'x', ha='right')
ax1.text(-1200, -1500, 'z')

# plot arrow representing dipole moment magnitude and direction
insetax = []
for ax in [ax1, ax2]:

    # plot arrow representing dipole moment magnitude and direction
    ax.annotate("", #r"$\mathbf{p}(I_n(t), \mathbf{r}_n)$",
                xy=(R_cell[0], R_cell[2]),
                xytext=(R_cell[0] + cell.current_dipole_moment[t_max][0, 0]*50,
                        R_cell[2] + cell.current_dipole_moment[t_max][0, 2]*50),
                arrowprops=dict(arrowstyle="<-", lw=3, color='k', shrinkA=0, shrinkB=0
                                ),
                zorder=100)
    ax.text(R_cell[0] + cell.current_dipole_moment[t_max][0, 0]*25 + 10,
             R_cell[2] + cell.current_dipole_moment[t_max][0, 2]*25,
             r"$\mathbf{p}$")
             # r"$\mathbf{p}(I_n^{(\mathrm{m})}(t), \mathbf{r}_n)$")


    # plot points where we show signal values
    ax.plot([0+R_cell[0]], [750+R_cell[2]], 'o', color='C0')
    ax.plot([-750+R_cell[0]], [0+R_cell[2]], 'o', color='C1')
    ax.plot([0+R_cell[0]], [-750+R_cell[2]], 'o', color='C2')
    ax.plot([750+R_cell[0]], [0+R_cell[2]], 'o', color='C3')


    bbox = np.array(ax.get_position())
    insetax.append(fig.add_axes([bbox[0, 0]+bbox[0,1]/4, 0.43, bbox[0,1]/4, 0.1]))



# Plot scalar y-component of magnetic field H at t=|i_syn|_max
vmax = 2.
mu = 4*np.pi*1E-7
im = ax2.pcolormesh(X_p, Z_p, H[t_max, :, 1].reshape(X_p.shape)*mu*1E12, # mT -> fT unit conversion
                  cmap=plt.get_cmap('BrBG', 51), zorder=-10,
                  vmin=-vmax,
                  vmax=vmax,
                  rasterized=True)

phi = np.linspace(0, 2*np.pi, 37)
r = 500
x = r*np.cos(phi) + R_cell[0]
y = r*np.sin(phi) + R_cell[2]
ax2.plot(x, y, ':w', lw=2)
ax2.text(R_cell[0], r+R_cell[2]+10, r'$\mathbf{B}_\mathbf{p}\cdot\mathbf{\hat{y}}$', ha='center', va='bottom', color='k')
ax2.text(R_cell[0], r+R_cell[2]-10, r'$\mathbf{B}\cdot\mathbf{\hat{y}}$', ha='center', va='top', color='k')

#ax2.text(R_cell[0], r+R_cell[2]-10, r'$\phi(I_n^{(\mathrm{m})})$', ha='center', va='top', color='w')



bbox = np.array(ax2.get_position())
cax = fig.add_axes([bbox[0][0]+(bbox[1][0]-bbox[0][0])/4, bbox[0][1], (bbox[1][0]-bbox[0][0])/2, 0.015])
axcb = fig.colorbar(im, cax=cax, orientation='horizontal')
axcb.outline.set_visible(False)
axcb.set_label(r'$\mathbf{B}\cdot\hat{\mathbf{y}}$ (fT)', labelpad=0)
axcb.set_ticks([-vmax, 0, vmax])

# scale bars
ax2.plot([800, 800], [-1650, -1550], 'k', lw=1, clip_on=False)
ax2.text(820, -1650, r'$100 \mu$m', fontsize=12)

# axis cross
ax2.annotate("", xy=(-1100, -1600), xycoords='data', xytext=(-900, -1600), textcoords='data',
             arrowprops=dict(arrowstyle="<|-", connectionstyle="arc3,rad=0", facecolor='black'))
ax2.annotate("", xy=(-1100, -1600), xycoords='data', xytext=(-1100, -1400), textcoords='data',
             arrowprops=dict(arrowstyle="<|-", connectionstyle="arc3,rad=0", facecolor='black'))
ax2.text(-900, -1700, 'x', ha='right')
ax2.text(-1200, -1500, 'z')


# plot in insetaxes the dipole moment potential and magnetic field magnitude
for i, (x, z) in enumerate(zip([R_cell[0], -750+R_cell[0], R_cell[0], 750+R_cell[0]],[750+R_cell[2], R_cell[2], -750+R_cell[2], R_cell[2]])):
    # ind = ((X_p == x) & (Z_p==z)).flatten()
    ind = (((X_p - x)**2 == ((X_p - x)**2).min()) & ((Z_p - z)**2 == ((Z_p - z)**2).min())).flatten()
    insetax[0].plot(cell.tvec, phi_p[:, ind]*1E6, 'C{}'.format(i), lw=1) # mV->nV
    insetax[1].plot(cell.tvec, H[:, ind, 1]*mu*1E12, 'C{}'.format(i), lw=1) # mT -> fT

insetax[0].set_ylabel(r'$\phi_\mathbf{p}(t)$ (nV)')
insetax[0].set_xticks([0, 5, 10])
insetax[0].set_xlabel('time (ms)')

insetax[1].set_ylabel(r'$\mathbf{B}_\mathbf{p}(t) \cdot \hat{\mathbf{y}}$ (fT)')
insetax[1].set_xticks([0, 5, 10])
insetax[1].set_xlabel('time (ms)')




# panel D. Illustration of 4-sphere volume conductor model geometry
# ax3.set_title('four-sphere volume conductor model')
for direction in ["xzero"]:
    ax3.axis[direction].set_visible(True)

for direction in ["left", "right", "bottom", "top"]:
    ax3.axis[direction].set_visible(False)


theta = np.linspace(0, np.pi, 31)

# draw some circles:
for i, r, label in zip(range(4), foursphereParams['radii'], ['brain', 'CSF', 'skull', 'scalp']):
    ax3.plot(np.cos(theta)*r, np.sin(theta)*r, 'C{}'.format(i), label=label + r', $r_%i=%i$ mm' % (i+1, r / 1000), clip_on=False)

# draw measurement points
ax3.plot(foursphereParams['r'][:, 0], foursphereParams['r'][:, 2], 'ko', label='EEG/MEG sites')
for i, (x, y, z) in enumerate(foursphereParams['r']):
    # theta = np.arcsin(x / foursphereParams['radii'][-1])
    # if x >= 0:
    #     ax3.text(x, z+5000, r'${}\pi$'.format(theta / np.pi))
    # else:
    #     ax3.text(x, z+5000, r'${}\pi$'.format(theta / np.pi), ha='right')
    ax3.text(x, z+2500, r'{}'.format(i + 1), ha='center')
    
# dipole location
ax3.plot([0], [dipole_position[-1]], 'k.', label='dipole site')
ax3.axis('equal')
ax3.set_xticks(np.r_[-np.array(foursphereParams['radii']), 0, foursphereParams['radii']])
ax3.set_xticklabels([])                       
ax3.legend(loc=(0.25, 0.15), frameon=False)



# four-sphere volume conductor
sphere = LFPy.FourSphereVolumeConductor(
    **foursphereParams
)
phi_p = sphere.calc_potential(cell.current_dipole_moment, rz=dipole_position)

# import example_parallel_network_plotting as plotting
vlimround = draw_lineplot(ax=ax4, data=phi_p*1E9, unit=r'pV', #mV -> pV unit conversion
                          dt=cell.dt, ztransform=False,
                          T=(0, cell.tstop), color='k', scalebarbasis='log10')      
# ax4.set_xticklabels([])
ax4.set_yticklabels([r'{}'.format(i + 1) for i in range(phi_p.shape[0])])
ax4.set_ylabel('position')
ax4.set_xlabel('time (ms)')




# 90 deg rotation matrices around x-, y- and z-axis
Rx90 = np.array([[1, 0, 0],[0, 0, -1],[0, 1, 0]])
Ry90 = np.array([[0, 0, 1],[0, 1, 0],[-1, 0, 0]])
Rz90 = np.array([[0, -1, 0],[1, 0, 0],[0, 0, 1]])

# compute the radial unit vector from the center of the sphere to each
# measurement point, then unit vectors along theta and phi
r_hat = (sphere.rxyz.T / sphere.r).T
theta = np.arccos(sphere.rxyz[:, 2] / sphere.r)
phi = np.arctan2(sphere.rxyz[:, 1], sphere.rxyz[:, 0])
theta_hat = np.array([np.cos(theta)*np.cos(phi),
                      np.cos(theta)*np.sin(phi),
                      -np.sin(phi)]).T
phi_hat = np.array([-np.sin(phi), np.cos(phi), np.zeros(r_hat.shape[0])]).T


ax5.set_title(r"scalp magnetic field $\mathbf{B}_\mathbf{p}(\mathbf{r}) \cdot \hat{\mathbf{\varphi}}$")
# radial component of H at squid locations
# create MEG object and compute magnetic field
meg = LFPy.MEG(sensor_locations= foursphereParams['r'])
H = meg.calculate_H(cell.current_dipole_moment, dipole_position)

H_phi = np.zeros(phi_p.shape)
for j, (h, u) in enumerate(zip(H, phi_hat)):
    H_phi[j, ] += np.dot(h, u)


vlimround = draw_lineplot(ax=ax5, data=H_phi*meg.mu*1E12, # mT -> fT unit conversion
                          dt=cell.dt, unit=r'fT',
                          ztransform=False,
                          label=r'$\mathbf{B}_\mathbf{p}(\mathbf{r}) \cdot \hat{\mathbf{\varphi}}$',
                          T=(0, cell.tstop), color='k', scalebarbasis='log10')      

ax5.set_yticklabels([r'{}'.format(i + 1) for i in range(phi_p.shape[0])])
ax5.set_xlabel('time (ms)', labelpad=0)
ax5.set_ylabel('')


for i, ax in enumerate([ax0, ax1, ax2]):
    ax.text(-0.05, 1.05, alphabet[i],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)
for i, ax in enumerate([ax3, ax4, ax5]):
    ax.text(-0.05, 1.1, alphabet[i+3],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)


fig.savefig('figure_2.pdf', bbox_inches='tight', dpi=300)
plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# cython: language_level=2


import numpy as np
cimport numpy as np
cimport cython

DTYPE = np.float64
ctypedef np.float64_t DTYPE_t
ctypedef Py_ssize_t   LTYPE_t

cdef extern from "math.h":
    int floor(DTYPE_t x)


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef np.ndarray[long, ndim=1, negative_indices=False] alias_method(np.ndarray[long, ndim=1, negative_indices=False] idx,
                 np.ndarray[DTYPE_t, ndim=1, negative_indices=False] probs,
                 int nsyn):
    """
    Alias method for drawing random numbers from a discrete probability
    distribution. See http://www.keithschwarz.com/darts-dice-coins/

    Parameters
    ----------
    idx : np.ndarray
        compartment indices as array of ints
    probs : np.ndarray
        compartment areas as array of floats
    nsyn : int
        number of randomized compartment indices

    Returns
    -------
    out : np.ndarray
        integer array of randomly drawn compartment indices

    """
    try:
        assert idx.size == probs.size
    except AssertionError as ae:
        raise ae('length of idx and probs arrays must be equal')

    #C-declare variables
    cdef np.ndarray[long, ndim=1, negative_indices=False] J, spc
    cdef np.ndarray[DTYPE_t, ndim=1, negative_indices=False] q
    cdef np.ndarray[DTYPE_t, ndim=2, negative_indices=False] rands
    cdef int nn, j, ad, K, kk

    # Construct the table.
    J, q = alias_setup(probs)

    #output array
    spc = np.zeros(nsyn, dtype=int)

    #prefetch random numbers, alias_draw needs nsyn x 2 numbers
    rands = np.random.rand(nsyn, 2)

    K = J.size
    # Generate variates using alias draw method
    for nn in range(nsyn):
        kk = floor(rands[nn, 0]*K)
        if rands[nn, 1] < q[kk]:
            spc[nn] = idx[kk]
        else:
            spc[nn] = idx[J[kk]]

    return spc


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef alias_setup(np.ndarray[DTYPE_t, ndim=1, negative_indices=False] probs):
    """Set up function for alias method.
    See http://www.keithschwarz.com/darts-dice-coins/

    Parameters
    ----------
    probs : np.ndarray
        float array of compartment areas

    Returns
    -------
    J : np.ndarray
        array of ints
    q : np.ndarray
        array of floats

    """
    #C-declare variables
    cdef np.ndarray[DTYPE_t, ndim=1, negative_indices=False] q
    cdef np.ndarray[long, ndim=1, negative_indices=False] J, smaller, larger
    cdef long K
    cdef int small, large, kk, s_i, l_i
    cdef DTYPE_t prob

    K = probs.size
    q = probs*K
    J = np.zeros(K, dtype=int)

    # Sort the data into the outcomes with probabilities
    # that are larger and smaller than 1/K.
    smaller = np.zeros(K, dtype=int)
    larger = np.zeros(K, dtype=int)
    s_i = 0
    l_i = 0
    for kk in range(K):
        if q[kk] < 1:
            smaller[s_i] = kk
            s_i += 1
        else:
            larger[l_i] = kk
            l_i += 1

    s_i -= 1
    l_i -= 1

    # Loop though and create little binary mixtures that
    # appropriately allocate the larger outcomes over the
    # overall uniform mixture.
    while s_i >= 0 and l_i >= 0:
        small = smaller[s_i]
        large = larger[l_i]

        J[small] = large
        q[large] = q[large] + q[small] - 1

        s_i -= 1

        if q[large] < 1:
            s_i += 1
            l_i -= 1
            smaller[s_i] = large

    return J, q



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''Defines classes and methods used by example_parallel_network.py script

Copyright (C) 2018 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
'''
from __future__ import division
import numpy as np
import json


def get_templatename(f):
    '''
    Assess from hoc file the templatename being specified within
    
    Arguments
    ---------
    f : file, mode 'r'
    
    Returns
    -------
    templatename : str
    
    '''    
    for line in f.readlines():
        if 'begintemplate' in line.split():
            templatename = line.split()[-1]
            print('template {} found!'.format(templatename))
            continue
    
    return templatename


def get_params(names, dictionary, keys, scaling=1.):
    '''
    generic function to extract data from e.g., pathways_anatomy
    dictionary object
    '''
    out = []
    for i, pre in enumerate(names):
        out.append([])
        for j, post in enumerate(names):
            out[i].append([])
            if type(keys) is str: # each entry just the value
                out[i][j] = dictionary['{}:{}'.format(pre, post)][keys]*scaling
            elif type(keys) is list: # each entry a dict w. keys loc scale
                assert len(keys) == 2
                out[i][j] = dict()
                for key, entry in zip(['loc', 'scale'], keys):
                    out[i][j][key] = dictionary['{}:{}'.format(pre, post)][entry]*scaling
    return out


def get_syn_params(shortnames, names, pathways_physiology, mtype_map, synapses_tsv):
    '''
    Extract from pathways_physiology and synapses_tsv connection stats
    between pre and postsynaptic populations for use with synapse model
    '''
    out = {}
    for i, (pre, pre_l) in enumerate(zip(shortnames, names)):
        for j, (post, post_l) in enumerate(zip(shortnames, names)):
            phys = pathways_physiology['{}:{}'.format(pre, post)]
            
            # use just the averaged values
            out['{}:{}'.format(pre, post)] = dict(Dep_mean = phys['d_mean'],
                             Dep_std = phys['d_std'],
                             Fac_mean = phys['f_mean'],
                             Fac_std = phys['f_std'],
                             )
            
            pre_mtype = mtype_map['pre_mtype_id'][mtype_map['pre_mtype'] == pre]
            data = synapses_tsv[post][synapses_tsv[post]['pre_mtype'] == pre_mtype]
            
            if type(data) is not np.void:
                if data.size > 1:
                    out['{}:{}'.format(pre, post)].update(dict(
                        Use_mean = data['use'].mean(),
                        Use_std = data['use'].std(),
                        tau_d_mean = data['tau_d'].mean(),
                        tau_d_std = data['tau_d'].std(),
                        delay_mean = data['delay'].mean(),
                        delay_std = data['delay'].std(),
                        weight_mean = data['weight'].mean(),
                        weight_std = data['weight'].std(),
                        synapse_type = data['synapse_type'][0], # I'm only gonna differentiate between excitatory
                                                                # (>= 100) and inhibitory (<100) connections.
                                                                # Connections can't be both. 
                        ))
                elif data.size == 1:
                    out['{}:{}'.format(pre, post)].update(dict(
                        Use_mean = data['use'],
                        Use_std = data['use'],
                        tau_d_mean = data['tau_d'],
                        tau_d_std = data['tau_d'],
                        delay_mean = data['delay'],
                        delay_std = data['delay'],
                        weight_mean = data['weight'],
                        weight_std = data['weight'],
                        synapse_type = data['synapse_type']
                        ))
                else: # presumably no connections are gonna be made anyway (bc. conn. prob. better be 0)
                    out['{}:{}'.format(pre, post)].update(dict(
                        Use_mean = 0,
                        Use_std = 0,
                        tau_d_mean = 0,
                        tau_d_std = 0,
                        delay_mean = 0,
                        delay_std = 0,
                        weight_mean = 0,
                        weight_std = 0,
                        synapse_type = 0
                        ))                                        
            else:
                if data.size == 1:
                    out['{}:{}'.format(pre, post)].update(dict(
                        Use_mean = data['use'],
                        Use_std = data['use'],
                        tau_d_mean = data['tau_d'],
                        tau_d_std = data['tau_d'],
                        delay_mean = data['delay'],
                        delay_std = data['delay'],
                        weight_mean = data['weight'],
                        weight_std = data['weight'],
                        synapse_type = data['synapse_type']
                        ))
                else:
                    out['{}:{}'.format(pre, post)].update(dict(
                        Use_mean = 0,
                        Use_std = 0.1,
                        tau_d_mean = 0,
                        tau_d_std = 0.1,
                        delay_mean = 1.5,
                        delay_std = 0.1,
                        weight_mean = 0,
                        weight_std = 0.1,
                        synapse_type = 0
                        ))                    
    return out


def get_L_yXL(fname, y, x_in_X, L, fill_empty_columns=True):
    '''
    compute the layer specificity, defined as: L_yXL = k_yXL / k_yX where
    k_yXL is the mean number of connections from presynaptic population X onto
    a cell in population y. 
        
    Parameters
    ----------
    fname : path
    y : str
        presynaptic cell type name
    x_in_X : list of list of str
        map postsynaptic cell type to postsynaptic population type
    L : list of str
        layer names
    fill_empty_columns : bool
        if True, empty columns is detected, and the value in the matrix
        corresponding to that column and "home" layer of the cell type wil be
        set to 1 (incoming connections may be assigned to that layer)
    '''
    def _get_L_yXL_per_yXL(fname, x_in_X, X_index,
                                  y, layer):
        # Load data from json dictionary
        f = open(fname, 'r')
        data = json.load(f)
        f.close()
    
        
        # Get number of synapses
        if layer in [str(key) for key in data['data'][y]['syn_dict'].keys()]:
            #init variables
            k_yXL = 0
            k_yX = 0
            
            for x in x_in_X[X_index]:
                p_yxL = data['data'][y]['syn_dict'][layer][x] / 100.
                k_yL = data['data'][y]['syn_dict'][layer]['number of synapses per neuron']
                k_yXL += p_yxL * k_yL
                
            for l in [str(key) for key in data['data'][y]['syn_dict'].keys()]:
                for x in x_in_X[X_index]:
                    p_yxL = data['data'][y]['syn_dict'][l][x] / 100.
                    k_yL = data['data'][y]['syn_dict'][l]['number of synapses per neuron']
                    k_yX +=  p_yxL * k_yL
            
            if k_yXL != 0.:
                return k_yXL / k_yX
            else:
                return 0.
        else:
            return 0.


    #init dict
    L_yXL = {}

    #iterate over postsynaptic cell types
    for y_value in y:
        #container
        data = np.zeros((len(L), len(x_in_X)))
        #iterate over lamina
        for i, Li in enumerate(L):
            #iterate over presynapse population inds
            for j in range(len(x_in_X)):
                data[i][j]= _get_L_yXL_per_yXL(fname, x_in_X,
                                                          X_index=j,
                                                          y=y_value,
                                                          layer=Li)
        
        # check if values should be assigned in empty columns
        if fill_empty_columns:
            mapping = (data != 0)
            for j, col in enumerate(mapping.sum(axis=0)):
                if col == 0:
                    for i, Li in enumerate(L):
                        if Li in y_value[:4]:
                            data[i,j] = 1.
        L_yXL[y_value] = data

    return L_yXL




#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''plotting script for figure 5 in manuscript preprint on output of
example_parallel_network.py

Copyright (C) 2018 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
'''
from __future__ import division
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec, GridSpecFromSubplotSpec
from mpl_toolkits.axisartist.axislines import SubplotZero
import os
import numpy as np
import h5py
from LFPy import FourSphereVolumeConductor, MEG
import example_parallel_network_plotting as plotting
from mpi4py import MPI

# set up MPI environment
COMM = MPI.COMM_WORLD
SIZE = COMM.Get_size()
RANK = COMM.Get_rank()

fontsize=14
titlesize=16
legendsize=12
plt.rcParams.update({
    'axes.xmargin': 0.0,
    'axes.ymargin': 0.0,
    'axes.labelsize': fontsize,
    'axes.titlesize': titlesize,
    'figure.titlesize': fontsize,
    'font.size': fontsize,
    'legend.fontsize': legendsize,
})

def plot_spike_raster(ax, PSET, T):
    f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
    for i, (m_name, name) in enumerate(zip(PSET.populationParameters['m_type'], PSET.populationParameters['me_type'])):
        x = []
        y = []
        ax.hlines(f['SPIKES'][name]['gids'][()].min(), T[0], T[1], 'k', lw=0.25)
        for gid, spt in zip(f['SPIKES'][name]['gids'], f['SPIKES'][name]['times']):
            if len(spt) > 0:
                y += [gid]*spt.size
                x += list(spt)
        x = np.array(x)
        y = np.array(y)
        inds = (x >= T[0]) & (x <= T[1])
        ax.plot(x[inds], y[inds], '|',
                color=colors[i], markersize=2, lw=2, clip_on=True, label=m_name)
    f.close()
    ax.set_xlim(T[0], T[1])
    ax.set_ylim(-0.5, PSET.populationParameters['POP_SIZE'].sum()+0.5)
    ax.invert_yaxis()
    ax.legend(loc=1, markerscale=5)
    ax.set_xlabel('time (ms)', labelpad=0)
    ax.set_ylabel('cell ID')
    ax.set_title('spike raster')
    
if __name__ == '__main__':
    # get simulation parameters
    from example_parallel_network_parameters import PSET

    # cell type colors 
    colors = [plt.get_cmap('Set1', PSET.populationParameters.size)(i) for i in range(PSET.populationParameters.size)]

    # time shown
    T = (PSET.TRANSIENT, PSET.TRANSIENT+1000.)

    # Set up figure and subplots 
    fig = plt.figure(figsize=(16, 15.5))
    gs = GridSpec(5, 3, hspace=0.3, wspace=0.2, left=0.06, right=0.95,
                  top=0.95, bottom=0.05)
    alphabet = 'ABCDEFGHIJKLMNOPQRST'
    
    # 90 deg rotation matrices around x-, y- and z-axis
    Rx90 = np.array([[1, 0, 0],[0, 0, -1],[0, 1, 0]])
    Ry90 = np.array([[0, 0, 1],[0, 1, 0],[-1, 0, 0]])
    Rz90 = np.array([[0, -1, 0],[1, 0, 0],[0, 0, 1]])

    
    # PANEL A. Spike raster
    ax = fig.add_subplot(gs[:2, 0])
    plot_spike_raster(ax, PSET, T)
    ax.set_xticklabels([])
    ax.set_xlabel('')
    plotting.remove_axis_junk(ax)
    ax.text(-0.1, 1.025, alphabet[0],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)


    # PANEL B. Spike count histogram
    gs0 = GridSpecFromSubplotSpec(4, 1, subplot_spec=gs[:2, 1])
    axes = [fig.add_subplot(gs0[i]) for i in range(PSET.populationParameters['me_type'].size)]
    
    f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
    
    dt = 5. # bin size for histograms
    bins = np.arange(T[0], T[1]+dt, dt)

    axes[0].set_title(r'spike-count histograms ($\Delta t={}$ ms)'.format(dt))
    
    for i, (m_name, name) in enumerate(zip(PSET.populationParameters['m_type'], PSET.populationParameters['me_type'])):
        ax = axes[i]
        plotting.remove_axis_junk(ax)
        data = np.hstack(f['SPIKES'][name]['times'][()].flat)
        ax.hist(data, bins=bins, color=colors[i][:-1], label=m_name)
        ax.axis(ax.axis('tight'))
        ax.set_xlim(PSET.TRANSIENT, PSET.TRANSIENT+1000.)
        ax.legend(loc=1)
        ax.set_ylabel('count')
        if ax != axes[-1]:
            ax.set_xticklabels([])
        else:
            ax.set_xlabel('time (ms)', labelpad=0)

    axes[0].text(-0.1, 1.1, alphabet[1],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=axes[0].transAxes)




    # PANEL C Extracellular potential
    gs0 = GridSpecFromSubplotSpec(12, 1, subplot_spec=gs[:2, 2])
    ax = fig.add_subplot(gs0[:-2])
    f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
    for data, title, color in zip(
                           [f['SUMMED_OUTPUT'][()]['imem']],
                           ['extracellular potentials, summed'],
                           ['k']):
        ax.set_title(title)
        vlimround = plotting.draw_lineplot(ax=ax, data=plotting.decimate(data, q=PSET.decimate_q),
                                  dt=PSET.dt*PSET.decimate_q,
                                  T=T, color=color, scalebarbasis='log10')
    f.close()

    ax.set_xticklabels([])
    ax.set_xlabel('')
    ax.set_ylabel('')
    ax.text(-0.1, 1.055, alphabet[2],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)



    # PANEL D ECoG potential
    ax = fig.add_subplot(gs0[-1])
    f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
    data = f['SUMMED_ECOG'][()]['imem']
    title = 'ECoG potential, summed'
    color = 'k'
    ax.set_title(title)
    vlimround = plotting.draw_lineplot(ax=ax,
                               data=plotting.decimate(f['SUMMED_OUTPUT'][()]['imem'][0, ].reshape((1, -1)),
                                                      q=PSET.decimate_q),
                              dt=PSET.dt*PSET.decimate_q,
                              scalebar=False,
                              T=T, color='0.5', scalebarpos=-1.5, scalebarbasis='log10')
    vlimround = plotting.draw_lineplot(ax=ax, data=plotting.decimate(data, q=PSET.decimate_q),
                              dt=PSET.dt*PSET.decimate_q,
                              vlimround=vlimround, scalebar=True,
                              T=T, color=color, scalebarpos=-1.5, scalebarbasis='log10')
    
    lines = ax.get_lines()
    ax.legend(('ch. 1', 'ECoG'), loc=8, ncol=2, bbox_to_anchor=(0.5, -1.25), frameon=False)
    
    f.close()
    ax.set_xticklabels([])
    ax.set_xlabel('')
    ax.set_ylabel('')
    ax.set_yticklabels([])
    ax.text(-0.1, 1.175, alphabet[3],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)

    


    # PANEL E. current dipole moment signal
    f = h5py.File(os.path.join(PSET.OUTPUTPATH,
                               'example_parallel_network_output.h5'), 'r')
    p_temp = np.zeros(f['CURRENT_DIPOLE_MOMENT'].shape)
    for name in f['CURRENT_DIPOLE_MOMENT'].dtype.names:
        p_temp += f['CURRENT_DIPOLE_MOMENT'][name]
    p_net = plotting.decimate(p_temp.T, q=PSET.decimate_q)
    p_net *=  1E-3 # nA um -> 1E-3 nA m unit conversion
    del p_temp

    gs0 = GridSpecFromSubplotSpec(3, 1, subplot_spec=gs[2, 0])
    for i, ylabel in enumerate([r'$\mathbf{p \cdot \hat{x}}$', r'$\mathbf{p \cdot \hat{y}}$', r'$\mathbf{p \cdot \hat{z}}$']):
        ax = fig.add_subplot(gs0[i])
        if i == 0:
            ax.set_title(r'current dipole moment ($10^{-3}$ nA m)')
            ax.text(-0.1, 1.15, alphabet[4],
                horizontalalignment='center',
                verticalalignment='center',
                fontsize=16, fontweight='demibold',
                transform=ax.transAxes)

        plotting.remove_axis_junk(ax)
        t = np.arange(p_net.shape[1])*PSET.dt*PSET.decimate_q
        inds = (t >= T[0]) & (t <= T[1])
        ax.plot(t[inds], p_net[i, inds], 'k', lw=1)
        ax.set_ylabel(ylabel)
        ax.set_xticklabels([])
        


    # panel F. Illustration of 4-sphere volume conductor model geometry
    ax = SubplotZero(fig, gs[2, 1])
    fig.add_subplot(ax)
    ax.set_title('four-sphere volume conductor model')

    for direction in ["xzero"]:
        ax.axis[direction].set_visible(True)

    for direction in ["left", "right", "bottom", "top"]:
        ax.axis[direction].set_visible(False)

    
    theta = np.linspace(0, np.pi, 31)
    
    # draw some circles:
    for i, r, label in zip(range(4), PSET.foursphereParams['radii'], ['brain', 'CSF', 'skull', 'scalp']):
        ax.plot(np.cos(theta)*r, np.sin(theta)*r, 'C{}'.format(i), label=label + r', $r_%i=%i$ mm' % (i+1, r / 1000), clip_on=False)
    
    # draw measurement points
    ax.plot(PSET.foursphereParams['r'][:, 0], PSET.foursphereParams['r'][:, 2], 'ko', label='EEG/MEG sites')
    
    for i, (x, y, z) in enumerate(PSET.foursphereParams['r']):
        ax.text(x, z+2500, r'{}'.format(i+1), ha='center')
        
    # dipole location
    ax.plot([0], [PSET.foursphereParams['radii'][0] + PSET.layer_data['center'][3]], 'k.', label='dipole site')
    
    ax.axis('equal')
    ax.set_ylim(top=max(PSET.foursphereParams['radii']) + 5000)

    ax.set_xticks(np.r_[-np.array(PSET.foursphereParams['radii']), 0, PSET.foursphereParams['radii']])
    ax.set_xticklabels([])                       
    
    ax.legend(loc=(0.25, 0.05), frameon=False)
    
    ax.text(-0.1, 1.05, alphabet[5],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)




    # PANEL G. EEG signal
    ax = fig.add_subplot(gs[2, 2])
    ax.set_title(r'surface potential $\phi_\mathbf{p}(\mathbf{r})$ ')

    f = h5py.File(os.path.join(PSET.OUTPUTPATH,
                               'example_parallel_network_output.h5'), 'r')
    
    # compute dipole potentials as the sum of contributions in
    # different positions
    phi_p = np.zeros((PSET.foursphereParams['r'].shape[0], f['CURRENT_DIPOLE_MOMENT'][name].shape[0]))
    for i, name in enumerate(PSET.populationParameters['me_type']):
        p = f['CURRENT_DIPOLE_MOMENT'][name]
        
        # four-sphere volume conductor
        sphere = FourSphereVolumeConductor(
            **PSET.foursphereParams
        )
        phi_p += sphere.calc_potential(p=p,
                                       rz=np.array([0, 0, PSET.foursphereParams['radii'][0] + PSET.layer_data['center'][3:][i % 2]]),)

    vlimround = plotting.draw_lineplot(ax=ax, data=plotting.decimate(phi_p, q=PSET.decimate_q)[::-1, ]*1E3, unit=r'$\mu$V', #mV -> µV unit conversion
                              dt=PSET.dt*PSET.decimate_q,
                              T=T, color='k', scalebarbasis='log10')

    ax.set_xticklabels([])
    ax.set_xlabel('')
    ax.set_yticklabels(['{}'.format(i+1) for i in range(sphere.rxyz.shape[0])])
    ax.set_ylabel('position', labelpad=10)

    ax.text(-0.1, 1.05, alphabet[6],
        horizontalalignment='center',
        verticalalignment='center',
        fontsize=16, fontweight='demibold',
        transform=ax.transAxes)




    # PANEL H. tangential component of MEG signal (as recorded by squid outside scull)
    
    # compute the radial unit vector from the center of the sphere to each
    # measurement point, then unit vectors along theta and phi
    r_hat = (sphere.rxyz.T / sphere.r).T
    theta = np.arccos(sphere.rxyz[:, 2] / sphere.r)
    phi = np.arctan2(sphere.rxyz[:, 1], sphere.rxyz[:, 0])
    theta_hat = np.array([np.cos(theta)*np.cos(phi),
                          np.cos(theta)*np.sin(phi),
                          -np.sin(phi)]).T
    phi_hat = np.array([-np.sin(phi), np.cos(phi), np.zeros(r_hat.shape[0])]).T
    
    for j, (unitvector, akse) in enumerate(zip([theta_hat, phi_hat, r_hat],
        [r'\hat{\mathbf{\theta}}', r'\hat{\mathbf{\varphi}}', r'\hat{\mathbf{r}}'])):
        ax = fig.add_subplot(gs[3, j])
        ax.set_title(r'surface magn. field $\mathbf{B}_\mathbf{p}(\mathbf{r}) \cdot %s$' % akse)
        
        # radial/tangential component of H at squid locations
        H_rt = np.zeros(phi_p.shape)
        for i, name in enumerate(PSET.populationParameters['me_type']):
            # dipole position
            dipole_position = np.array([0, 0, PSET.foursphereParams['radii'][0] + PSET.layer_data['center'][3:][i % 2]])
            # create MEG object and compute magnetic field
            meg = MEG(sensor_locations = PSET.foursphereParams['r'])
            H = meg.calculate_H(f['CURRENT_DIPOLE_MOMENT'][name], dipole_position)
            
            for k, (h, u) in enumerate(zip(H, unitvector)):
                H_rt[k, ] += np.dot(h, u)
            
        B_rt = H_rt * meg.mu # unit mT (from nA/µm * Tm/A)
        vlimround = plotting.draw_lineplot(ax=ax, data=plotting.decimate(B_rt, q=PSET.decimate_q)[::-1, ]*1E12, # mT --> fT unit conversion
                                  dt=PSET.dt*PSET.decimate_q, unit=r'fT',
                                  T=T, color='k', scalebarbasis='log10')
        
        ax.set_yticklabels(['{}'.format(i+1) for i in range(sphere.rxyz.shape[0])])
        ax.set_xlabel('')
        ax.set_xticklabels([])
        if j == 0:
            ax.set_ylabel('position', labelpad=10)
            ax.text(-0.1, 1.05, alphabet[7],
                horizontalalignment='center',
                verticalalignment='center',
                fontsize=16, fontweight='demibold',
                transform=ax.transAxes)
        else:
            ax.set_yticklabels([])
            ax.set_ylabel('')

        

        # PANEL I. tangential components of MEG signal (as recorded by squid outside scull)
        # with dipole sources rotated 90 deg counterclockwise around x-axis
        ax = fig.add_subplot(gs[4, j])
        ax.set_title(r'surface magn. field $\mathbf{B}_{R_x(\pi/2)\mathbf{p}}(\mathbf{r}) \cdot %s$' % akse)

        # radial/tangential component of H at squid locations
        H_rt = np.zeros(phi_p.shape)
        for i, name in enumerate(PSET.populationParameters['me_type']):
            # dipole position
            dipole_position = np.array([0, 0, PSET.foursphereParams['radii'][0] + PSET.layer_data['center'][3:][i % 2]])
            # create MEG object and compute magnetic field
            meg = MEG(sensor_locations= PSET.foursphereParams['r'])
            H = meg.calculate_H(np.dot(Rx90, f['CURRENT_DIPOLE_MOMENT'][name].T).T, dipole_position)
            # compute the radial unit vector from the center of the sphere to each
            # measurement point

            for k, (h, u) in enumerate(zip(H, unitvector)):
                H_rt[k, ] += np.dot(h, u)
                
        B_rt = H_rt * meg.mu # unit mT (from nA/µm * Tm/A)
        vlimround = plotting.draw_lineplot(ax=ax, data=plotting.decimate(B_rt, q=PSET.decimate_q)[::-1, ]*1E12, # mT --> fT unit conversion
                                  dt=PSET.dt*PSET.decimate_q, unit=r'fT',
                                  T=T, color='k', scalebarbasis='log10')
    
        ax.set_yticklabels(['{}'.format(i+1) for i in range(sphere.rxyz.shape[0])])
        ax.set_xlabel('time (ms)', labelpad=0)
        if j == 0:
            ax.set_ylabel('position', labelpad=10)
            ax.text(-0.1, 1.05, alphabet[8],
                horizontalalignment='center',
                verticalalignment='center',
                fontsize=16, fontweight='demibold',
                transform=ax.transAxes)
        else:
            ax.set_yticklabels([])
            ax.set_ylabel('')

    fig.savefig(os.path.join(PSET.OUTPUTPATH, 'figure_5.pdf'), bbox_inches='tight')

    plt.show()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''plotting/analysis routines on output of example_parallel_network.py

Copyright (C) 2018 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
'''
from __future__ import division
import matplotlib
# matplotlib.use('agg')
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
from matplotlib.collections import PolyCollection
import os
import numpy as np
import scipy.signal as ss
import h5py
from LFPy import NetworkCell
from mpi4py import MPI
import neuron

# set up MPI environment
COMM = MPI.COMM_WORLD
SIZE = COMM.Get_size()
RANK = COMM.Get_rank()

# set default plotting parameters
fontsize=14
titlesize=16
legendsize=12
plt.rcParams.update({
    'axes.xmargin': 0.0,
    'axes.ymargin': 0.0,
    'axes.labelsize': fontsize,
    'axes.titlesize': titlesize,
    'figure.titlesize': fontsize,
    'font.size': fontsize,
    'legend.fontsize': legendsize,
})

def decimate(x, q=10, n=4, k=0.8, filterfun=ss.cheby1):
    """
    scipy.signal.decimate like downsampling using filtfilt instead of lfilter,
    and filter coeffs from butterworth or chebyshev type 1.


    Parameters
    ----------
    x : numpy.ndarray
        Array to be downsampled along last axis.
    q : int 
        Downsampling factor.
    n : int
        Filter order.
    k : float
        Aliasing filter critical frequency Wn will be set as Wn=k/q.
    filterfun : function
        `scipy.signal.filter_design.cheby1` or
        `scipy.signal.filter_design.butter` function

    Returns
    -------
    numpy.ndarray
        Array of downsampled signal.
              
    """
    if not isinstance(q, int):
        raise TypeError("q must be an integer")

    if n is None:
        n = 1

    if filterfun == ss.butter:
        b, a = filterfun(n, k / q)
    elif filterfun == ss.cheby1:
        b, a = filterfun(n, 0.05, k / q)
    else:
        raise Exception('only ss.butter or ss.cheby1 supported')

    try:
        y = ss.filtfilt(b, a, x)
    except: # Multidim array can only be processed at once for scipy >= 0.9.0
        y = []
        for data in x:
            y.append(ss.filtfilt(b, a, data))
        y = np.array(y)

    try:
        return y[:, ::q]
    except:
        return y[::q]


def draw_lineplot(
        ax, data, dt=0.1,
        T=(0, 200),
        scaling_factor=1.,
        vlimround=None,
        label='local',
        scalebar=True,
        scalebarpos=0,
        scalebarbasis='log2',
        unit='mV',
        ylabels=True,
        color='r',
        ztransform=True,
        filter=False,
        filterargs=dict(N=2, Wn=0.02, btype='lowpass')):
    ''' draw some nice lines'''
    
    tvec = np.arange(data.shape[1])*dt
    try:
        tinds = (tvec >= T[0]) & (tvec <= T[1])
    except TypeError:
        print(data.shape, T)
        raise Exception
    
    # apply temporal filter
    if filter:
        b, a = ss.butter(**filterargs)
        data = ss.filtfilt(b, a, data, axis=-1)
    
    #subtract mean in each channel
    if ztransform:
        dataT = data.T - data.mean(axis=1)
        data = dataT.T

    zvec = -np.arange(data.shape[0])
    vlim = abs(data[:, tinds]).max()
    if vlimround is None:
        vlimround = 2.**np.round(np.log2(vlim)) / scaling_factor
    else:
        pass

    yticklabels=[]
    yticks = []
    
    for i, z in enumerate(zvec):
        if i == 0:
            ax.plot(tvec[tinds], data[i][tinds] / vlimround + z, lw=1,
                    rasterized=False, label=label, clip_on=False,
                    color=color)
        else: 
            ax.plot(tvec[tinds], data[i][tinds] / vlimround + z, lw=1,
                    rasterized=False, clip_on=False,
                    color=color)
        yticklabels.append('ch. %i' % (i+1))
        yticks.append(z)
     
    if scalebar:
        if scalebarbasis == 'log2':
            ax.plot([tvec[tinds][-1], tvec[tinds][-1]],
                    [-1-scalebarpos, -2-scalebarpos], lw=2, color=color, clip_on=False)
            ax.text(tvec[tinds][-1]+np.diff(T)*0.03, -1.5-scalebarpos,
                    '$2^{' + '{}'.format(int(np.log2(vlimround))) + '}$ ' + '{0}'.format(unit),
                    color=color, rotation='vertical',
                    va='center')
        elif scalebarbasis == 'log10':
            # recompute scale bar size to show it on scientific format
            vlimround10 = 10**np.round(np.log10(vlimround))
            if vlimround10 >= 1:
                vlimround10 = int(np.round(vlimround10))
            rescale = vlimround10 / vlimround
            ax.plot([tvec[tinds][-1], tvec[tinds][-1]],
                    np.array([0.5, -0.5])*rescale - 1.5 - scalebarpos,
                    lw=2, color=color, clip_on=False)
            ax.text(tvec[tinds][-1]+np.diff(T)*0.03, -1.5-scalebarpos,
                    '{0} '.format(vlimround10) + '{0}'.format(unit),
                    color=color, rotation='vertical',
                    va='center')

    ax.axis(ax.axis('tight'))
    ax.yaxis.set_ticks(yticks)
    if ylabels:
        ax.yaxis.set_ticklabels(yticklabels)
        ax.set_ylabel('channel', labelpad=0.1)
    else:
        ax.yaxis.set_ticklabels([])
    remove_axis_junk(ax, lines=['right', 'top'])
    ax.set_xlabel(r'time (ms)', labelpad=0.1)
    
    return vlimround


def remove_axis_junk(ax, lines=['right', 'top']):
    for loc, spine in ax.spines.items():
        if loc in lines:
            spine.set_color('none')            
    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.set_ticks_position('left')


def plot_connectivity(ax, PSET, cmap=plt.get_cmap('inferno'),
                      data='connprob', cbarlabel=r'$C_{YX}$'):

    '''make an imshow of the intranetwork connectivity'''
    masked_array = np.ma.array(PSET.connParams[data], mask=np.array(PSET.connParams[data])==0.)
    cmap.set_bad('k', 0.5)
    im = ax.pcolormesh(masked_array, cmap=cmap, vmin=0, ) #interpolation='nearest')
    ax.axis(ax.axis('tight'))
    ax.invert_yaxis()
    ax.xaxis.set_ticks_position('top')
    ax.set_xticks(np.arange(PSET.populationParameters.size)+0.5)
    ax.set_yticks(np.arange(PSET.populationParameters.size)+0.5)
    ax.set_xticklabels(PSET.populationParameters['m_type'], rotation=270)
    ax.set_yticklabels(PSET.populationParameters['m_type'], )
    ax.xaxis.set_label_position('top')
    ax.set_xlabel(r'$Y$', labelpad=0)
    ax.set_ylabel(r'$X$', labelpad=0, rotation=0)

    rect = np.array(ax.get_position().bounds)
    rect[0] += rect[2] + 0.0025
    rect[2] = 0.005
    fig = plt.gcf()
    cax = fig.add_axes(rect)

    cbar = plt.colorbar(im, cax=cax)
    cbar.set_label(cbarlabel, labelpad=0)
    

def plot_quantity_yXL(fig, left, bottom, top, PSET, quantity,
                      y=['p23', 'b23', 'nb23', 'p4', 'ss4(L23)', 'ss4(L4)',
                         'b4', 'nb4', 'p5(L23)', 'p5(L56)', 'b5', 'nb5',
                         'p6(L4)', 'p6(L56)', 'b6', 'nb6'],
                      label=r'$\mathcal{L}_{yXL}$', layers = ['L1', 'L2/3', 'L4', 'L5', 'L6'],
                      cmap=plt.get_cmap('inferno')):
                            
    '''make a bunch of image plots, each showing the spatial normalized
    connectivity of synapses'''
    
 
    
    ncols = 3 #int(np.floor(np.sqrt(len(y))))
    nrows = int(len(y) // ncols)
    if len(y) % ncols > 0:
        nrows += 1

    
    #assess vlims
    vmin = 0
    vmax = 0
    for yi in y:
        if quantity[yi].max() > vmax:
            vmax = quantity[yi].max()
    
    gs = GridSpec(nrows, ncols, left=left, bottom=bottom, top=top)
    
    for i, yi in enumerate(y):
        ax = fig.add_subplot(gs[i//ncols, i%ncols])

        masked_array = np.ma.array(quantity[yi], mask=quantity[yi]==0)
        
        im = ax.pcolormesh(masked_array,
                            vmin=vmin, vmax=vmax,
                            cmap=cmap,
                            )
        ax.invert_yaxis()

        ax.axis(ax.axis('tight'))
        ax.xaxis.set_ticks_position('top')
        ax.set_xticks(np.arange(len(y))+0.5)
        ax.set_yticks(np.arange(len(layers))+0.5)
        
        if i % ncols == 0:
            ax.set_yticklabels(layers, )
            ax.set_ylabel('$L$', labelpad=0.)
        else:
            ax.set_yticklabels([])
        if i < ncols:
            ax.set_xlabel(r'$X$', labelpad=-1,fontsize=8)
            ax.set_xticklabels(y, rotation=90)
        else:
            ax.set_xticklabels([])
        ax.xaxis.set_label_position('top')
        
        ax.text(0.5, -0.13, r'$y=$'+yi,
            horizontalalignment='center',
            verticalalignment='center',
            #
                transform=ax.transAxes,fontsize=5.5)
    
        #colorbar
        if (i // ncols==0) and (i % ncols) == ncols-1:
            rect = np.array(ax.get_position().bounds)
            rect[0] += rect[2] + 0.01
            rect[1] = bottom
            rect[2] = 0.01
            rect[3] = top-bottom
            cax = fig.add_axes(rect)
            cbar = plt.colorbar(im, cax=cax)
            cbar.set_label(label, labelpad=0)


    
def plot_m_types(ax, PSET, colors, section=['dend', 'apic'], spacing=300, linewidths=0.05):
    '''draw comparison plot of each individual morphology'''
    CWD=PSET.CWD
    CELLPATH=PSET.CELLPATH
    n_segs = []
    areas = []

    for i, data in enumerate(PSET.populationParameters):
        
        NRN = data["me_type"]
        os.chdir(os.path.join(CWD, CELLPATH, NRN))
        cell = NetworkCell(**PSET.cellParameters[NRN])
        cell.set_pos(x=i*spacing, y=0, z=data['pop_args']['loc'])
        cell.set_rotation(x=np.pi/2)
        print(NRN, cell.zstart.min(), cell.zmid.min(), cell.zend.min(), cell.zstart.max(), cell.zmid.max(), cell.zend.max())   
        n_segs += [cell.totnsegs]
        areas += [cell.area[cell.get_idx(section)].sum()]

        zips = []
        for x, z in cell.get_idx_polygons(projection=('x', 'z')):
            zips.append(list(zip(x, z)))

        polycol = PolyCollection(zips,
                                 edgecolors=colors[i],
                                 linewidths=linewidths,
                                 facecolors=colors[i],
                                 label=NRN,
                                 )
        ax.add_collection(polycol)
        os.chdir(CWD)
    
    axis = ax.axis(ax.axis('tight'))

    # draw lines showing the layer boundaries
    ax.hlines(np.r_[0., -PSET.layer_data['thickness'].cumsum()][:4], axis[0], axis[1]-300, 'k', lw=0.5)
    ax.hlines(np.r_[0., -PSET.layer_data['thickness'].cumsum()][4:], axis[0], axis[1], 'k', lw=0.5)
    
    # annotate hlines with values
    for z in np.r_[0., -PSET.layer_data['thickness'].cumsum()]:
        ax.text(axis[0], z, r'$z={}$'.format(int(z)) + '$\mu$m', ha='right', va='center')
    
    ax.set_yticks(PSET.layer_data['center'])
    ax.set_yticklabels(PSET.layer_data['layer'])
    
    ax.set_xticks(np.arange(PSET.populationParameters.size)*spacing)
    ax.set_xticklabels(PSET.populationParameters['m_type'], rotation='vertical')

    ax.axis(ax.axis('equal'))
    ax.set_title('m-types')
    neuron.h("forall delete_section()")
    
    return n_segs, areas
            

if __name__ == '__main__':    
    # get simulation parameters
    from example_parallel_network_parameters import PSET


    ############################################################################
    # Plot simulated output
    ############################################################################

    if not os.path.isdir(PSET.OUTPUTPATH):
        if RANK == 0:
            os.mkdir(PSET.OUTPUTPATH)
    COMM.Barrier()

    ############################################
    T = (PSET.TRANSIENT, PSET.tstop)
    colors = [plt.get_cmap('Set1', PSET.populationParameters.size)(i) for i in range(PSET.populationParameters.size)]
    ############################################



    # plot m-types in network
    fig, ax = plt.subplots(1, 1, figsize=(PSET.populationParameters.size, 10))
    plot_m_types(ax, PSET, colors, spacing=300.)
    fig.savefig(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_m_types.pdf'),
                bbox_inches='tight')
    plt.close(fig)


    # plot connection probabilities between pre and postsynaptic populations
    fig, ax = plt.subplots(1,1)
    fig.subplots_adjust(top=0.85)
    plot_connectivity(ax, PSET)
    fig.savefig(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_connectivity.pdf'),
                bbox_inches='tight')
    plt.close(fig)
    

    # plot layer specificity of connections between pre and postsynaptic cell types
    fig = plt.figure()
    fig.suptitle('layer specificity of connections')
    plot_quantity_yXL(fig=fig, left=0.1, bottom=0.05, top=0.8, PSET=PSET,
                      quantity=PSET.L_YXL_m_types,
                      y=PSET.populationParameters['m_type'],
                      layers = PSET.layer_data['layer'],
                      label=r'$\mathcal{L}_{YXL}$')
    fig.savefig(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_L_YXL.pdf'),
                        bbox_inches='tight')
    plt.close(fig)

    
    # plot summed LFP and contributions of leak and capacitive currents
    if RANK == 0 and PSET.COMPUTE_LFP:
        f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
        for data, title, suffix, color in zip(
                               [#f['SUMMED_OUTPUT'].value['imem'],
                                # f['SUMMED_OUTPUT'].value['ipas'],
                                # f['SUMMED_OUTPUT'].value['icap'],
                                # f['SUMMED_OUTPUT'].value['isyn_e'],
                                # f['SUMMED_OUTPUT'].value['isyn_i'],
                                # f['SUMMED_OUTPUT'].value['isyn_e'] + f['SUMMED_OUTPUT'].value['isyn_i'],                                
                                # f['SUMMED_OUTPUT'].value['imem'] - f['SUMMED_OUTPUT'].value['ipas'] - f['SUMMED_OUTPUT'].value['icap'] - f['SUMMED_OUTPUT'].value['isyn_e'] - f['SUMMED_OUTPUT'].value['isyn_i'],
                                ] + [f['SUMMED_OUTPUT'].value[name] for name in f['SUMMED_OUTPUT'].dtype.names],
                               [#'extracellular potentials, summed',
                                # 'extracellular potential, leak currents',
                                # 'extracellular potential, capacitive currents',
                                # 'extracellular potential, exc. synaptic currents',
                                # 'extracellular potential, inh. synaptic currents',
                                # 'extracellular potential, exc. + inh. synaptic currents',
                                # 'extracellular potential, residual',
                                ] + [name for name in f['SUMMED_OUTPUT'].dtype.names],
                               [#'LFP',
                                # 'i_pas', 'i_cap', 'i_syn_e', 'i_syn_i', 'i_syn_ei', 'i_gX'
                                ] + [name for name in f['SUMMED_OUTPUT'].dtype.names],
                               [#'k',
                                # 'r', 'b', 'c', 'm', 'g', 'y'
                                ] + [colors[i] for i in range(PSET.populationParameters.size)]):
            fig = plt.figure(figsize=(20, 10)) 
            ax = fig.add_subplot(121)    
            ax.set_title(title)
            vlimround = draw_lineplot(ax=ax, data=decimate(data, q=PSET.decimate_q),
                                      dt=PSET.dt*PSET.decimate_q,
                                      T=T, color=color)      

            ax = fig.add_subplot(122)    
            ax.set_title(title + r' (LP filtered, $f_\mathrm{crit}=100$ Hz)')
            vlimround = draw_lineplot(ax=ax, data=decimate(data, q=PSET.decimate_q),
                                      dt=PSET.dt*PSET.decimate_q,
                                      T=T, color=color,
                                      ztransform=True, filter=True,
                                      filterargs=PSET.filterargs)      

            # save figure output
            fig.savefig(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_summed_{}.pdf'.format(suffix)),
                        bbox_inches='tight')
            plt.close(fig)
        f.close()
            

    if RANK == 0 and PSET.COMPUTE_LFP:
        fig = plt.figure(figsize=(10, 10))
        ax = fig.add_subplot(211)
        ax.set_title('extracellular signal variance')
        y = PSET.electrodeParams['z']
        yticklabels = ['ch. {}'.format(x+1) for x in range(y.size)]
        tind = int(PSET.TRANSIENT / PSET.dt)        
        f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
        for data, label, color in zip([#f['SUMMED_OUTPUT'].value['imem'],
                                       # f['SUMMED_OUTPUT'].value['ipas'],
                                       # f['SUMMED_OUTPUT'].value['icap'],
                                       # f['SUMMED_OUTPUT'].value['isyn_e'],
                                       # f['SUMMED_OUTPUT'].value['isyn_i'],
                                       # f['SUMMED_OUTPUT'].value['isyn_e'] + f['SUMMED_OUTPUT'].value['isyn_i'],
                                       # f['SUMMED_OUTPUT'].value['imem'] - f['SUMMED_OUTPUT'].value['ipas'] - f['SUMMED_OUTPUT'].value['icap'] - f['SUMMED_OUTPUT'].value['isyn_e'] - f['SUMMED_OUTPUT'].value['isyn_i']
                                       ] + [f['SUMMED_OUTPUT'].value[name] for name in f['SUMMED_OUTPUT'].dtype.names],
                                      [#'sum',
                                       # r'$i_\mathrm{pas}$', r'$i_\mathrm{cap}$', r'$i_\mathrm{syn, E}$', r'$i_\mathrm{syn, I}$', r'$i_\mathrm{syn, E}+i_\mathrm{syn, I}$', 'residual'
                                       ] + [name for name in f['SUMMED_OUTPUT'].dtype.names],
                                      [#'k',
                                       # 'r', 'b', 'c', 'm', 'g', 'y'
                                       ] + ['k'] + [colors[i] for i in range(PSET.populationParameters.size)]):
            ax.semilogx(data[:, tind:].var(axis=1), y, lw=2, label=label, color=color)
        f.close()
        ax.set_yticks(y)
        ax.set_yticklabels(yticklabels)
        ax.axis(ax.axis('tight'))
        ax.legend(loc='best')
        ax.set_xlabel(r'variance (mV$^2$)')


        ax = fig.add_subplot(212)
        ax.set_title(r'LP filtered signals ($f_\mathrm{crit}=100$ Hz, 4th order Butterworth, filtfilt)')
        b, a = ss.butter(**PSET.filterargs)
        f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
        for data, label, color in zip([#f['SUMMED_OUTPUT'].value['imem'],
                                       # f['SUMMED_OUTPUT'].value['ipas'],
                                       # f['SUMMED_OUTPUT'].value['icap'],
                                       # f['SUMMED_OUTPUT'].value['isyn_e'],
                                       # f['SUMMED_OUTPUT'].value['isyn_i'],
                                       # f['SUMMED_OUTPUT'].value['isyn_e'] + f['SUMMED_OUTPUT'].value['isyn_i'],
                                       # f['SUMMED_OUTPUT'].value['imem'] - f['SUMMED_OUTPUT'].value['ipas'] - f['SUMMED_OUTPUT'].value['icap'] - f['SUMMED_OUTPUT'].value['isyn_e'] - f['SUMMED_OUTPUT'].value['isyn_i']
                                       ] + [f['SUMMED_OUTPUT'].value[name] for name in f['SUMMED_OUTPUT'].dtype.names],
                                      [#'sum',
                                       # r'$i_\mathrm{pas}$', r'$i_\mathrm{cap}$', r'$i_\mathrm{syn, E}$', r'$i_\mathrm{syn, I}$', r'$i_\mathrm{syn, E}+i_\mathrm{syn, I}$', 'residual'
                                       ] + [name for name in f['SUMMED_OUTPUT'].dtype.names],
                                      [#'k',
                                       # 'r', 'b', 'c', 'm', 'g', 'y'
                                       ] + ['k'] + [colors[i] for i in range(PSET.populationParameters.size)]):
            ax.semilogx(ss.filtfilt(b, a, data, axis=-1)[:, tind:].var(axis=1), y, lw=2, label=label, color=color)
        f.close()
        ax.set_yticks(y)
        ax.set_yticklabels(yticklabels)
        ax.axis(ax.axis('tight'))
        ax.set_xlabel(r'variance (mV$^2$)')
        
        fig.savefig(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_variance.pdf'),
                    bbox_inches='tight')
        plt.close(fig)
    
    
    # spike raster plot of all spiking activity from file
    if RANK == 0:
        fig, ax = plt.subplots(1,1, figsize=(10, 10))
        f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
        for i, name in enumerate(PSET.populationParameters['me_type']):
            x = []
            y = []
            ax.hlines(f['SPIKES'][name]['gids'].value.min(), T[0], T[1], 'k', lw=0.25)
            for gid, spt in zip(f['SPIKES'][name]['gids'], f['SPIKES'][name]['times']):
                if len(spt) > 0:
                    y += [gid]*spt.size
                    x += list(spt)
            ax.plot(x, y, '|',
                    color=colors[i], markersize=2, lw=2, clip_on=True, label=name)
        f.close()
        ax.axis(ax.axis('tight'))
        ax.set_xlim(PSET.TRANSIENT, PSET.tstop)
        ax.set_ylim(ax.axis()[2]-0.5, ax.axis()[3]+0.5)
        ax.invert_yaxis()
        ax.legend(loc='best')
        ax.set_xlabel('time (ms)')
        ax.set_ylabel('gid')
        ax.set_title('spike raster')
        
        # save figure output
        fig.savefig(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_raster.pdf'),
                    bbox_inches='tight')
        plt.close(fig)


    # spike count rate histograms of all spiking activity from file
    if RANK == 0:
        fig, axes = plt.subplots(PSET.populationParameters.size, 1,
                                 figsize=(10, 10), sharex=True)
        fig.subplots_adjust(left=0.2)
        f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')

        dt = 10. # bin size for histograms
        bins = np.arange(T[0], T[1]+dt, dt)

        axes[0].set_title(r'population spike time histogram, ($\Delta t={}$ ms)'.format(dt))
        
        for i, name in enumerate(PSET.populationParameters['me_type']):
            ax = axes[i]
            data = np.hstack(f['SPIKES'][name]['times'].value.flat)
            ax.hist(data, bins=bins, color=colors[i]) #, histtype='step', color=colors[i])
            ax.axis(ax.axis('tight'))
            ax.set_xlim(PSET.TRANSIENT, PSET.tstop)
            ax.set_ylabel(name, rotation='horizontal', labelpad=50)
            if ax != axes[-1]:
                ax.set_xticklabels([])
            else:
                ax.set_xlabel('time (ms)')

        f.close()
        
        # save figure output
        fig.savefig(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_spike_time_histogram.pdf'),
                    bbox_inches='tight')
        plt.close(fig)
        
    
    # spike count histogram across populations from file
    if RANK == 0:
        n = PSET.populationParameters['me_type'].size
        ncols = int(np.floor(np.sqrt(n)))
        nrows = int(np.ceil(float(n) / ncols ))
        gs = GridSpec(nrows, ncols)
        fig = plt.figure(figsize=(10, 10))
        fig.subplots_adjust(hspace=0.4)
        fig.suptitle('per-cell spike count hist., T={} s'.format((PSET.tstop - PSET.TRANSIENT) / 1000.))
        
        bins = np.arange(42) * (PSET.tstop - PSET.TRANSIENT) / 1000. # make count bins conform to bin size of 1 Hz. 
        f = h5py.File(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_output.h5'), 'r')
        for i, name in enumerate(PSET.populationParameters['me_type']):
            ax = fig.add_subplot(gs[i // ncols, i % ncols])
            x = []
            for spt in f['SPIKES'][name]['times']:
                if spt.size == 0:
                    x += [0]                
                else:
                    if np.any(spt >= PSET.TRANSIENT):
                        x += [spt[spt >= PSET.TRANSIENT].size]
                    else:
                        x += [0]
            ax.hist(x, bins=bins, color=colors[i], clip_on=True, label=name) # histtype='step', 
            ax.axis(ax.axis('tight'))
            ax.set_title(name)
            
            if i >= (n - ncols):
                ax.set_xlabel('spike count')
            else:
                ax.set_xticklabels([])
            if i % ncols == 0:
                ax.set_ylabel('observations')
        f.close()
        # save figure output
        fig.savefig(os.path.join(PSET.OUTPUTPATH, 'example_parallel_network_spike_count_hist.pdf'),
                    bbox_inches='tight')
        plt.close(fig)        



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Some plottin' functions used by the example scripts

Copyright (C) 2017 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
"""

from __future__ import division
import numpy as np
import matplotlib.pyplot as plt
import neuron

plt.rcParams.update({
        'axes.xmargin': 0.0,
        'axes.ymargin': 0.0,
    })

def plot_ex1(cell, electrode, X, Y, Z):
    '''
    plot the morphology and LFP contours, synaptic current and soma trace
    '''
    #some plot parameters
    t_show = 30 #time point to show LFP
    tidx = np.where(cell.tvec == t_show)
    #contour lines:
    n_contours = 200
    n_contours_black = 40
    
    #This is the extracellular potential, reshaped to the X, Z mesh
    LFP = np.arcsinh(electrode.LFP[:, tidx]).reshape(X.shape)
    
    #figure object
    fig = plt.figure(figsize=(12, 8))
    fig.subplots_adjust(left=None, bottom=None, right=None, top=None, 
        wspace=0.4, hspace=0.4)
    
    # Plot LFP around the cell with in color and with equipotential lines
    ax1 = fig.add_subplot(121, aspect='equal', frameon=False)
    
    #plot_morphology(plot_synapses=True)
    for sec in neuron.h.allsec():
        idx = cell.get_idx(sec.name())
        ax1.plot(np.r_[cell.xstart[idx], cell.xend[idx][-1]],
                np.r_[cell.zstart[idx], cell.zend[idx][-1]],
                color='k')
    for i in range(len(cell.synapses)):
        ax1.plot([cell.synapses[i].x], [cell.synapses[i].z], '.',
            #color=cell.synapses[i].color, marker=cell.synapses[i].marker, 
            markersize=10)
    
    #contour lines
    ct1 = ax1.contourf(X, Z, LFP, n_contours)
    ct1.set_clim((-0.00007, 0.00002))
    ct2 = ax1.contour(X, Z, LFP, n_contours_black, colors='k')
    
    # Plot synaptic input current
    ax2 = fig.add_subplot(222)
    ax2.plot(cell.tvec, cell.synapses[0].i)
    
    # Plot soma potential
    ax3 = fig.add_subplot(224)
    ax3.plot(cell.tvec, cell.somav)
    
    # Figure formatting and labels
    fig.suptitle('example 1', fontsize=14)
    
    ax1.set_title('LFP at t=' + str(t_show) + ' ms', fontsize=12)
    ax1.set_xticks([])
    ax1.set_xticklabels([])
    ax1.set_yticks([])
    ax1.set_yticklabels([])
    
    ax2.set_title('synaptic input current', fontsize=12)
    ax2.set_ylabel('(nA)')
    ax2.set_xlabel('time (ms)')
    
    ax3.set_title('somatic membrane potential', fontsize=12)
    ax3.set_ylabel('(mV)')
    ax3.set_xlabel('time (ms)')
    
    return fig


def plot_ex2(cell, electrode):
    '''example2.py plotting function'''
    #creating array of points and corresponding diameters along structure
    for i in range(cell.xend.size):
        if i == 0:
            xcoords = np.array([cell.xmid[i]])
            ycoords = np.array([cell.ymid[i]])
            zcoords = np.array([cell.zmid[i]])
            diams = np.array([cell.diam[i]])    
        else:
            if cell.zmid[i] < 100 and cell.zmid[i] > -100 and \
                    cell.xmid[i] < 100 and cell.xmid[i] > -100:
                xcoords = np.r_[xcoords,
                                np.linspace(cell.xstart[i],
                                            cell.xend[i], int(cell.length[i]*3))]   
                ycoords = np.r_[ycoords,
                                np.linspace(cell.ystart[i],
                                            cell.yend[i], int(cell.length[i]*3))]   
                zcoords = np.r_[zcoords,
                                np.linspace(cell.zstart[i],
                                            cell.zend[i], int(cell.length[i]*3))]   
                diams = np.r_[diams,
                              np.linspace(cell.diam[i], cell.diam[i],
                                          int(cell.length[i]*3))]
    
    #sort along depth-axis
    argsort = np.argsort(ycoords)
    
    #plotting
    fig = plt.figure(figsize=[15, 10])
    ax = fig.add_axes([0.1, 0.1, 0.533334, 0.8], frameon=False)
    ax.scatter(xcoords[argsort], zcoords[argsort], s=diams[argsort]**2*20,
               c=ycoords[argsort], edgecolors='none', cmap='gray')
    ax.plot(electrode.x, electrode.z, '.', marker='o', markersize=5, color='k')
    
    i = 0
    limLFP = abs(electrode.LFP).max()
    for LFP in electrode.LFP:
        tvec = cell.tvec*0.6 + electrode.x[i] + 2
        if abs(LFP).max() >= 1:
            factor = 2
            color='r'
        elif abs(LFP).max() < 0.25:
            factor = 50
            color = 'b'
        else:
            factor = 10
            color = 'g'
        trace = LFP*factor + electrode.z[i]
        ax.plot(tvec, trace, color=color, lw = 2)
        i += 1
    
    ax.plot([22, 28], [-60, -60], color='k', lw = 3)
    ax.text(22, -65, '10 ms')
    
    ax.plot([40, 50], [-60, -60], color='k', lw = 3)
    ax.text(42, -65, '10 $\mu$m')
    
    ax.plot([60, 60], [20, 30], color='r', lw=2)
    ax.text(62, 20, '5 mV')
    
    ax.plot([60, 60], [0, 10], color='g', lw=2)
    ax.text(62, 0, '1 mV')
    
    ax.plot([60, 60], [-20, -10], color='b', lw=2)
    ax.text(62, -20, '0.1 mV')
    
    
    
    ax.set_xticks([])
    ax.set_yticks([])
    
    ax.axis([-61, 61, -61, 61])
    
    ax.set_title('Location-dependent extracellular spike shapes')
    
    #plotting the soma trace    
    ax = fig.add_axes([0.75, 0.55, 0.2, 0.35])
    ax.plot(cell.tvec, cell.somav)
    ax.set_title('Somatic action-potential')
    ax.set_ylabel(r'$V_\mathrm{membrane}$ (mV)')
    
    #plotting the synaptic current
    ax = fig.add_axes([0.75, 0.1, 0.2, 0.35])
    ax.plot(cell.tvec, cell.synapses[0].i)
    ax.set_title('Synaptic current')
    ax.set_ylabel(r'$i_\mathrm{synapse}$ (nA)')
    ax.set_xlabel(r'time (ms)')

    return fig



def plot_ex3(cell, electrode):
    '''plotting function used by example3/4'''
    fig = plt.figure(figsize=[12, 8])
    
    #plot the somatic trace
    ax = fig.add_axes([0.1, 0.7, 0.5, 0.2])
    ax.plot(cell.tvec, cell.somav)
    ax.set_xlabel('Time (ms)')
    ax.set_ylabel('Soma pot. (mV)')
    
    #plot the synaptic current
    ax = fig.add_axes([0.1, 0.4, 0.5, 0.2])
    for i in range(len(cell.synapses)):
        ax.plot(cell.tvec, cell.synapses[i].i,
                color='C0' if cell.synapses[i].kwargs['e'] > cell.v_init else 'C1',
                )
    ax.set_xlabel('Time (ms)')
    ax.set_ylabel('Syn. i (nA)')
    
    #plot the LFP as image plot
    ax = fig.add_axes([0.1, 0.1, 0.5, 0.2])
    absmaxLFP = electrode.LFP.std()*3 #abs(np.array([electrode.LFP.max(), electrode.LFP.min()])).max()
    im = ax.pcolormesh(cell.tvec, electrode.z, electrode.LFP,
                  vmax=absmaxLFP, vmin=-absmaxLFP,
           cmap='PRGn')
    
    rect = np.array(ax.get_position().bounds)
    rect[0] += rect[2] + 0.01
    rect[2] = 0.02
    cax = fig.add_axes(rect)
    cbar = plt.colorbar(im, cax=cax)
    cbar.set_label('LFP (mV)')
    ax.axis(ax.axis('tight'))
    ax.set_xlabel('Time (ms)')
    ax.set_ylabel('z ($\mu$m)')
    
    #plot the morphology, electrode contacts and synapses
    ax = fig.add_axes([0.65, 0.1, 0.25, 0.8], frameon=False)
    for sec in neuron.h.allsec():
        idx = cell.get_idx(sec.name())
        ax.plot(np.r_[cell.xstart[idx], cell.xend[idx][-1]],
                np.r_[cell.zstart[idx], cell.zend[idx][-1]],
                color='k')
    for i in range(len(cell.synapses)):
        ax.plot([cell.synapses[i].x], [cell.synapses[i].z], marker='.',
            color='C0' if cell.synapses[i].kwargs['e'] > cell.v_init else 'C1',
        )
    for i in range(electrode.x.size):
        ax.plot(electrode.x[i], electrode.z[i], color='C2', marker='o')
    # plt.axis(np.array(plt.axis())*0.8)
    ax.set_xticks([])
    ax.set_yticks([])
    
    return fig




import LFPy
import neuron #new
import numpy as np
import pickle
import os
import matplotlib.pyplot as plt

def simulate_cells_serially(stimolo, \
                cell_ids, \
                data_name,\
                population_parameters, \
                cell_parameters, \
                synapse_parameters,\
                synapse_position_parameters,\
                electrode_parameters):

    print('input'+str(stimolo))

    # Load emitted spikes, 1st column: spike time, 2nd column: pre cell id
#    print "Loading locally emitted spikes"
    local_spikes_filename = population_parameters['input_dir']+'spiketimes_'+str(stimolo)+'.1.out'
    local_spikes = np.loadtxt(local_spikes_filename)
    local_sp_times = local_spikes[:,0]
    local_sp_ids = local_spikes[:,1]

    # Load connectivity, 1st column post id, 2nd column pre id
    connectivity_filename = population_parameters['input_dir']+'Cmatrix_'+str(stimolo)+'.1.out'
    connectivity_file = open(connectivity_filename, "r")
    lines = connectivity_file.readlines()
    incoming_connections = []
    for line in lines:
        incoming_connections.append(np.array(line.split(),dtype='int'))
    connectivity_file.close()

    pre_cells = {}
    pre_cells['exc_exc'] = population_parameters['exc_ids']
    pre_cells['exc_inh'] = population_parameters['exc_ids']
    pre_cells['inh_exc'] = population_parameters['inh_ids']
    pre_cells['inh_inh'] = population_parameters['inh_ids']

    n_thalamic_synapses = population_parameters['n_thalamic_synapses']
    n_external_synapses = population_parameters['n_external_synapses']

    # setup data dictionary

    output_data = {}
    output_data['somav'] = {}
    output_data['LFP'] = {}
    output_data['somapos'] = {}
    output_data['tot_isyn'] = {}


    for i_cell,cell_id in enumerate(cell_ids):

        if cell_id in population_parameters['exc_ids']:
            print(str(cell_id))
            cell_parameters.update({'morphology':'pyr1.hoc'})
            cell_parameters['passive_parameters'].update({'g_pas':1./20000.})

        elif cell_id in population_parameters['inh_ids']:
            indin=int(cell_id-max(population_parameters['exc_ids']))
            print(str(indin))
            cell_parameters.update({'morphology':'int1.hoc'})
            cell_parameters['passive_parameters'].update({'g_pas':1./10000.})


        print("Setting up cell "+str(cell_id))

        cell_seed = population_parameters['global_seed']+cell_id

        print("Setting random seed: "+str(cell_seed))

        np.random.seed(cell_seed)

        neuron.h('forall delete_section()') #new
        cell = LFPy.Cell(**cell_parameters) #new

        # load true position
        if cell_id in population_parameters['exc_ids']:
            cell_pos=np.loadtxt('PCsXYZ.txt')
            x,y,z=cell_pos[cell_id]

        elif cell_id in population_parameters['inh_ids']:
            cell_pos=np.loadtxt('INTsXYZ.txt')
            x,y,z=cell_pos[int(cell_id-int(min(population_parameters['inh_ids'])))]

        cell.set_pos(x=x,y=y,z=z)

        if cell_id in population_parameters['exc_ids']:
            local_synapse_types = ['exc_exc','inh_exc']
            thalamic_synapse_type = 'thalamic_exc'
            external_synapse_type = 'external_exc'

        elif cell_id in population_parameters['inh_ids']:
            local_synapse_types = ['exc_inh','inh_inh']
            thalamic_synapse_type = 'thalamic_inh'
            external_synapse_type = 'external_inh'

        for synapse_type in local_synapse_types:

            print("Setting up local synapses: ",synapse_type)

            pre_ids = incoming_connections[cell_id]
            n_synapses = len(pre_ids)

            for i_synapse, pre_id in enumerate(pre_ids):
                if pre_id in pre_cells[synapse_type]:
                    syn_idx = int(cell.get_rand_idx_area_norm(\
                                **synapse_position_parameters[synapse_type]))
                    synapse_parameters[synapse_type].update({'idx':syn_idx})
                    synapse = LFPy.Synapse(cell,\
                                        **synapse_parameters[synapse_type])
                    spike_times =\
                        local_sp_times[np.where(local_sp_ids==pre_id)[0]]
                    synapse.set_spike_times(spike_times)

        print("Setting up thalamic synapses")

        # Load thalamic input spike times, 1st column time, 2nd column post cell id
        #   print "Loading thalamic input spikes"
#         thalamic_spikes_filename = population_parameters['input_dir']+'ths/th_'+str(stimolo)+'_'+str(cell_id)+'.out'

#         print thalamic_spikes_filename
#
#         thalamic_spike_times = np.loadtxt(thalamic_spikes_filename)

#         synapse_ids =\
#                  np.random.randint(0,n_thalamic_synapses,len(thalamic_spike_times))

#         for i_synapse in xrange(n_thalamic_synapses):
#             syn_idx = int(cell.get_rand_idx_area_norm(\
#                         **synapse_position_parameters[thalamic_synapse_type]))
#             synapse_parameters[thalamic_synapse_type].update({'idx':syn_idx})
#             synapse = LFPy.Synapse(cell, \
#                             **synapse_parameters[thalamic_synapse_type])
#             spike_times = \
#                     thalamic_spike_times[np.where(synapse_ids==i_synapse)[0]]
#             synapse.set_spike_times(spike_times)

        print("Setting up external synapses")

        # Load external cortico-cortical input rate
        #  print "Loading external input spikes"
#         external_spikes_filename = population_parameters['input_dir']+'ccs/cc_'+str(stimolo)+'_'+str(cell_id)+'.out'
#         external_spike_times = np.loadtxt(external_spikes_filename)
#
#         synapse_ids =\
#              np.random.randint(0,n_external_synapses,len(external_spike_times))
#
#         for i_synapse in xrange(n_external_synapses):
#             syn_idx = int(cell.get_rand_idx_area_norm(\
#                         **synapse_position_parameters[external_synapse_type]))
#
#             synapse_parameters[external_synapse_type].update({'idx':syn_idx})
#             synapse = LFPy.Synapse(cell,\
#                             **synapse_parameters[external_synapse_type])
#             spike_times =\
#                 external_spike_times[np.where(synapse_ids==i_synapse)[0]]
#             synapse.set_spike_times(spike_times)


        # Run simulation
        print("Running simulation...")
        cell.simulate(rec_imem=True)

        # Calculate LFP
        print("Calculating LFP")
        electrode = LFPy.RecExtElectrode(cell,**electrode_parameters)
        electrode.calc_lfp()

        # Store data
        print("Storing data")
        output_data['LFP'][cell_id] = electrode.LFP
        output_data['somav'][cell_id] = cell.somav
        output_data['somapos'][cell_id] = cell.somapos


    output_data['tvec']=cell.tvec

    print("Saving data to file")

    if not os.path.isdir(population_parameters['save_to_dir']):
        os.mkdir(population_parameters['save_to_dir'])
    print(output_data)
    pickle.dump( output_data, open( \
            population_parameters['save_to_dir']+data_name+str(stimolo), "wb" ) )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Copyright (C) 2012 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""


import numpy as np
import neuron

class PointProcess(object):
    """
    Superclass on top of Synapse, StimIntElectrode, just to import and set
    some shared variables and extracts Cartesian coordinates of a segment
    
    Parameters
    ----------
    cell : obj
        LFPy.Cell object
    idx : int
        index of segment
    record_current : bool
        Must be set to True for recording of pointprocess currents
    record_potential : bool
        Must be set to True for recording potential of pointprocess target idx
    kwargs : pointprocess specific variables passed on to cell/neuron

    """
    def __init__(self, cell, idx, record_current=False, record_potential=False, **kwargs):
        """
        Initializes the PointProcess class
        """
        for key in ['color', 'marker']:
            if key in kwargs.keys():
                raise DeprecationWarning('Parameter {} has been deprecated'.format(key))
        self.idx = idx
        self.record_current = record_current
        self.record_potential = record_potential
        self.kwargs = kwargs
        self.update_pos(cell)

    def update_pos(self, cell):
        """
        Extract coordinates of point-process 
        """
        self.x = cell.xmid[self.idx]
        self.y = cell.ymid[self.idx]
        self.z = cell.zmid[self.idx]


class Synapse(PointProcess):
    """
    The synapse class, pointprocesses that spawn membrane currents.
    See http://www.neuron.yale.edu/neuron/static/docs/help/neuron/neuron/mech.html#pointprocesses
    for details, or corresponding mod-files.
    
    This class is meant to be used with synaptic mechanisms, giving rise to
    currents that will be part of the membrane currents. 

    Parameters
    ----------
    cell : obj
        `LFPy.Cell` or `LFPy.TemplateCell` instance to receive synapptic
        input
    idx : int
        Cell index where the synaptic input arrives
    syntype : str
        Type of synapse. Built-in examples: ExpSyn, Exp2Syn
    record_current : bool
        Decides if current is recorded
    **kwargs
        Additional arguments to be passed on to
        NEURON in `cell.set_synapse`

    Examples
    --------
    >>> import pylab as pl
    >>> pl.interactive(1)
    >>> import LFPy
    >>> import os
    >>> cellParameters = {
    >>>     'morphology' :  os.path.join('examples', 'morphologies', 'L5_Mainen96_LFPy.hoc'),
    >>>     'passive' : True,
    >>>     'tstop' :     50,
    >>> }
    >>> cell = LFPy.Cell(**cellParameters)

    >>> synapseParameters = {
    >>>     'idx' : cell.get_closest_idx(x=0, y=0, z=800),
    >>>     'e' : 0,                                # reversal potential
    >>>     'syntype' : 'ExpSyn',                   # synapse type
    >>>     'tau' : 2,                              # syn. time constant
    >>>     'weight' : 0.01,                        # syn. weight
    >>>     'record_current' : True                 # syn. current record
    >>> }
    >>> synapse = LFPy.Synapse(cell, **synapseParameters)
    >>> synapse.set_spike_times(pl.array([10, 15, 20, 25]))
    >>> cell.simulate()

    >>> pl.subplot(211)
    >>> pl.plot(cell.tvec, synapse.i)
    >>> pl.title('Synapse current (nA)')
    >>> pl.subplot(212)
    >>> pl.plot(cell.tvec, cell.somav)
    >>> pl.title('Somatic potential (mV)')

    """
    def __init__(self, cell, idx, syntype, record_current=False, record_potential=False, **kwargs):
        """
        Initialization of class Synapse
        """
        PointProcess.__init__(self, cell, idx, record_current, record_potential, **kwargs)
            
        self.syntype = syntype
        self.cell = cell
        self.hocidx = int(cell.set_synapse(idx=idx, syntype=syntype,
                                           record_current=record_current,
                                           record_potential=record_potential,
                                           **kwargs))
        self._ns_index = int(cell.netstimlist.count()) - 1
        cell.synapses.append(self)
        cell.synidx.append(idx)
        self.cell.sptimeslist.append(np.array([]))

    def set_spike_times(self, sptimes=np.zeros(0)):
        """Set the spike times explicitly using numpy arrays"""
        try:
            assert type(sptimes) is np.ndarray
        except AssertionError:
            raise AssertionError('synapse activation times must be a np.ndarray, not type({})'.format(type(sptimes)))
        self.cell.sptimeslist.insrt(self._ns_index, sptimes)
        self.cell.sptimeslist.remove(self._ns_index + 1)
    
    def set_spike_times_w_netstim(self, noise=1., start=0., number=1E3,
                                  interval=10., seed=1234.):
        """
        Generate a train of pre-synaptic stimulus times by setting up the
        neuron NetStim object associated with this synapse
        
        Parameters
        ----------
        noise : float in range [0, 1]
            Fractional randomness, from deterministic to intervals that drawn
            from negexp distribution (Poisson spiketimes).
        start : float
            ms, (most likely) start time of first spike
        number : int
            (average) number of spikes
        interval : float
            ms, (mean) time between spikes
        seed : float
            Random seed value
        """
        self.cell.netstimlist[self._ns_index].noise = noise
        self.cell.netstimlist[self._ns_index].start = start
        self.cell.netstimlist[self._ns_index].number = number
        self.cell.netstimlist[self._ns_index].interval = interval        
        self.cell.netstimlist[self._ns_index].seed(seed)

    def collect_current(self, cell):
        """Collect synapse current"""
        try:
            self.i = np.array(cell.synireclist.o(self.hocidx))
        except:
            raise Exception('cell.synireclist deleted from consequtive runs')
    
    def collect_potential(self, cell):
        """Collect membrane potential of segment with synapse"""
        try:
            self.v = np.array(cell.synvreclist.o(self.hocidx))
        except:
            raise Exception('cell.synvreclist deleted from consequtive runs')


class StimIntElectrode(PointProcess):
    """Class for NEURON point processes representing electrode currents,
    such as VClamp, SEClamp and ICLamp.
    
    Membrane currents will no longer sum to zero if these mechanisms are used,
    as the equivalent circuit is akin to a current input to the compartment
    from a far away extracellular location ("ground"), not immediately from
    the surface to the inside of the compartment as with transmembrane currents.
    
    Refer to NEURON documentation @ neuron.yale.edu for keyword arguments or 
    class documentation in Python issuing e.g.
        
        help(neuron.h.VClamp)
    
    Will insert pptype on cell-instance, pass the corresponding kwargs onto
    cell.set_point_process.

    Parameters
    ----------
    cell : obj
        `LFPy.Cell` or `LFPy.TemplateCell` instance to receive Stimulation
         electrode input
    idx : int
        Cell segment index where the stimulation electrode is placed
    pptype : str
        Type of point process. Built-in examples: VClamp, SEClamp and ICLamp.
        Defaults to 'SEClamp'.
    record_current : bool
        Decides if current is recorded
    record_potential : bool
        switch for recording the potential on postsynaptic segment index
    **kwargs
        Additional arguments to be passed on to
        NEURON in `cell.set_point_process`

    Examples
    --------
    >>> import pylab as pl
    >>> pl.ion()
    >>> import os
    >>> import LFPy
    >>> #define a list of different electrode implementations from NEURON
    >>> pointprocesses = [
    >>>     {
    >>>         'idx' : 0,
    >>>         'record_current' : True,
    >>>         'pptype' : 'IClamp',
    >>>         'amp' : 1,
    >>>         'dur' : 20,
    >>>         'delay' : 10,
    >>>     },
    >>>     {
    >>>         'idx' : 0,
    >>>         'record_current' : True,
    >>>         'pptype' : 'VClamp',
    >>>         'amp[0]' : -70,
    >>>         'dur[0]' : 10,
    >>>         'amp[1]' : 0,
    >>>         'dur[1]' : 20,
    >>>         'amp[2]' : -70,
    >>>         'dur[2]' : 10,
    >>>    },
    >>>    {
    >>>        'idx' : 0,
    >>>        'record_current' : True,
    >>>        'pptype' : 'SEClamp',
    >>>        'dur1' : 10,
    >>>        'amp1' : -70,
    >>>        'dur2' : 20,
    >>>        'amp2' : 0,
    >>>        'dur3' : 10,
    >>>        'amp3' : -70,
    >>>     },
    >>>  ]
    >>>  #create a cell instance for each electrode
    >>> for pointprocess in pointprocesses:
    >>>      cell = LFPy.Cell(morphology=os.path.join('examples', 'morphologies', 'L5_Mainen96_LFPy.hoc'),
    >>>                      passive=True)
    >>>      stimulus = LFPy.StimIntElectrode(cell, **pointprocess)
    >>>      cell.simulate()
    >>>      pl.subplot(211)
    >>>      pl.plot(cell.tvec, stimulus.i, label=pointprocess['pptype'])
    >>>      pl.legend(loc='best')
    >>>      pl.title('Stimulus currents (nA)')
    >>>      pl.subplot(212)
    >>>      pl.plot(cell.tvec, cell.somav, label=pointprocess['pptype'])
    >>>      pl.legend(loc='best')
    >>>      pl.title('Somatic potential (mV)')

    """    
    def __init__(self, cell, idx, pptype='SEClamp',
                 record_current=False,
                 record_potential=False, **kwargs):
        """Initialize StimIntElectrode class"""
        PointProcess.__init__(self, cell=cell, idx=idx,
                              record_current=record_current,
                              record_potential=record_potential)
        self.pptype = pptype
        self.hocidx = int(cell.set_point_process(idx, pptype,
                                                 record_current=record_current,
                                                 record_potential=record_potential,
                                                 **kwargs))
        cell.pointprocesses.append(self)
        cell.pointprocess_idx.append(idx)

    def collect_current(self, cell):
        """Fetch electrode current from recorder list"""
        self.i = np.array(cell.stimireclist.o(self.hocidx))

    def collect_potential(self, cell):
        """Collect membrane potential of segment with PointProcess"""
        self.v = np.array(cell.stimvreclist.o(self.hocidx))

